@@ -33,20 +33,18 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
-#include "include/forms.h"
-#include "flinternal.h"
-#include "fd_main.h"
-
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 
-static FRM *forms = NULL;   /* The forms */
-int fnumb = 0;
+#include "fd_main.h"
+
+static FRM *forms = NULL;   /* The list of forms */
+static int fnumb = 0;       /* number of forms */
 
 FL_FORM *cur_form = NULL;   /* The current form */
 
@@ -68,7 +66,23 @@ get_form_numb( FL_FORM * form )
 
 
 /***************************************
- * Sets the current form to numb, to NULL when numb = -1
+ ***************************************/
+
+const char *
+get_form_name( FL_FORM * form )
+{
+    int i;
+
+    for ( i = 0; i < fnumb; i++ )
+        if ( forms[ i ].form == form )
+            return *forms[ i ].fname ? forms[ i ].fname : NULL;
+
+    return NULL;
+}
+
+
+/***************************************
+ * Sets the current form to 'numb' or to NULL when 'numb' is -1
  ***************************************/
 
 static void
@@ -84,12 +98,16 @@ set_form( int numb )
         cur_form = forms[ numb ].form;
         set_bounding_box( 0, 0, cur_form->w, cur_form->h );
         fl_select_browser_line( fd_control->formbrowser, numb + 1 );
-        fl_winstepunit( main_window, 1, 1 );
+        fl_winstepsize( main_window, 1, 1 );
         fl_winresize( main_window, cur_form->w, cur_form->h );
         if ( fl_display )
             XSync( fl_display, 0 );
     }
 
+    fl_deselect_browser( fd_control->objectbrowser );
+    reset_pallette( );
+    cur_class = -1;
+
     clear_selection( );
     fillin_groups( );
     redraw_the_form( 1 );
@@ -97,7 +115,7 @@ set_form( int numb )
 
 
 /***************************************
- * Change current form background. Called when the main window is resized
+ * Change the current forms background. Called when the main window is resized
  ***************************************/
 
 void
@@ -143,14 +161,21 @@ void
 addform_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
             long        arg  FL_UNUSED_ARG )
 {
-    float w = 0,
-          h = 0;
+    double w = 0,
+           h = 0;
     FL_Coord xx,
              yy;
     static int form_seq;
     const char *s;
     const char *sp;
     FRM *new_forms;
+    int cc = cur_class;
+    static int busy = 0;
+
+    if ( busy )
+        return;
+    else
+        busy = 1;
 
     new_forms = fl_realloc( forms, ( fnumb + 1 ) * sizeof *forms );
 
@@ -158,6 +183,7 @@ addform_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
     {
         fl_show_alert( "Too many forms", "Running out of memory for forms",
                        NULL, 0 );
+        busy = 0;
         return;
     }
     else
@@ -169,8 +195,6 @@ addform_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
 
     /* Get boundary */
 
-    fl_deactivate_form( fd_control->control );
-
     fl_get_win_size( main_window, &xx, &yy );
     set_bounding_box( 0, 0, xx, yy );
 
@@ -183,38 +207,32 @@ addform_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
                                    "a C variable):", "" ) )
          || ! *s )
     {
-        fl_activate_form( fd_control->control );
+        busy = 0;
         return;
     }
 
     if (    ! isascii( ( unsigned char ) *s )
          || ! ( isalpha( ( unsigned char ) *s ) || *s == '_' ) )
     {
-        fl_show_alert( "Error", "Invalid C identifier specified for form "
-                       "name:", s, 0 );
+        fl_show_alert( "Error", "Invalid C identifier for form name:", s, 0 );
         goto get_new_form_name;
     }
 
-    sp = s + 1;
-
-    while ( *sp )
-    {
+    for ( sp = s + 1; *sp; sp++ )
         if (    ! isascii( ( unsigned char ) *sp )
-             || ! ( isalnum( ( unsigned char ) *sp ) || *s == '_' ) )
+             || ! ( isalnum( ( unsigned char ) *sp ) || *sp == '_' ) )
         {
-            fl_show_alert( "Error", "Invalid C identifier specified for form "
-                           "name:", s, 0 );
+            fl_show_alert( "Error", "Invalid C identifier for form name:",
+                           s, 0 );
             goto get_new_form_name;
         }
-        sp++;
-    }
 
     /* Create the form */
 
     cur_form = forms[ fnumb ].form = fl_bgn_form( FL_NO_BOX, w, h );
     fl_end_form( );
 
-    add_an_object( FL_BOX, FL_UP_BOX, 0, 0, w, h );
+    add_an_object( FL_BOX, FL_FLAT_BOX, 0, 0, w, h );
     fl_set_form_dblbuffer( cur_form, 1 );
 
     /* Get form name and add it */
@@ -225,16 +243,21 @@ addform_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
         sprintf( forms[ fnumb ].fname, "form%d", form_seq++ );
     fl_add_browser_line( fd_control->formbrowser, forms[ fnumb ].fname );
 
-    /* finish off */
+    /* Finish off */
 
     set_form( fnumb++ );
-    fl_activate_form( fd_control->control );
     changed = FL_TRUE;
+
+    if ( cc >= 0 )
+        select_object_by_class( cc );
+
+    busy = 0;
 }
 
 
 /***************************************
  * Callback routine invoked when the user wants to change the name
+ * of the current form
  ***************************************/
 
 void
@@ -242,8 +265,9 @@ changename_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
                long        arg  FL_UNUSED_ARG )
 {
     int fn = get_form_numb( cur_form );
-    const char *s;
-    const char *sp;
+    const char *s, *cn;
+    int i;
+    FL_OBJECT *o;
 
     if ( cur_form == NULL || fn == -1 )
         return;
@@ -255,28 +279,33 @@ changename_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
          || ! *s )
         return;
 
-    if (    ! isascii( ( unsigned char ) *s )
-         || ! ( isalpha( ( unsigned char ) *s ) || *s == '_' ) )
+    if ( ! is_valid_c_name( s ) )
     {
-        fl_show_alert( "Error", "Invalid C identifier specified for form "
-                       "name:", s, 0 );
+        fl_show_alert( "Error", "Invalid C identifier for form name:", s, 0 );
         goto get_changed_form_name;
     }
 
-    sp = s + 1;
-
-    while ( *sp )
+    for ( i = 0; i < fnumb; i++ )
     {
-        if (    ! isascii( ( unsigned char ) *sp )
-             || ! ( isalnum( ( unsigned char ) *sp ) || *s == '_' ) )
+        if ( i == fn )
+            continue;
+
+        if ( ! strcmp( forms[ i ].fname, s ) )
         {
-            fl_show_alert( "Error", "Invalid C identifier specified for form "
-                           "name:", s, 0 );
+            fl_show_alert( "Error", "New name is already in use for another "
+                           "form", NULL, 0 );
             goto get_changed_form_name;
         }
-        sp++;
     }
 
+    for ( o = forms[ fn ].form->first; o; o = o->next )
+        if ( ( cn = get_object_c_name( o ) ) && ! strcmp( s, cn ) )
+        {
+            fl_show_alert( "Error", "New name is already used for one of the ",
+                           "forms objects", 0 );
+            goto get_changed_form_name;
+        }
+
     fli_sstrcpy( forms[ fn ].fname, s, MAX_VAR_LEN );
 
     fl_replace_browser_line( fd_control->formbrowser, fn + 1,
@@ -354,7 +383,7 @@ changesize_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
 
 
 /***************************************
- * Callback routine called to delete a form
+ * Callback routine for deleting a form
  ***************************************/
 
 void
@@ -366,16 +395,21 @@ deleteform_cb( FL_OBJECT * obj  FL_UNUSED_ARG,
 
     if ( cur_form == NULL || fn == -1 )
         return;
+
     if ( ! fl_show_question( "Delete current form?", 1 ) )
         return;
+
     fl_delete_browser_line( fd_control->formbrowser, fn + 1 );
+
     for ( i = fn; i < fnumb - 1; i++ )
         forms[ i ] = forms[ i + 1 ];
+
     fnumb--;
     set_form( -1 );
-    changed = 1;
+    changed = FL_TRUE;
 }
 
+
 /****
   DRAWING FORMS
 ****/
@@ -450,23 +484,24 @@ load_fd_header( void )
          || strcmp( p, "Magic" )
          || ff_read( "%d", &fd_magic ) <= 0
          || (    fd_magic != MAGIC2 && fd_magic != MAGIC3
-              && fd_magic != MAGIC4 && fd_magic != MAGIC5 ) )
+              && fd_magic != MAGIC4 && fd_magic != MAGIC5
+              && fd_magic != MAGIC6 && fd_magic != MAGIC7 ) )
         return ff_err( "Wrong type of file" );
 
-    if ( fd_magic != MAGIC5 )
+    if ( fd_magic < MAGIC7 )
     {
         char *tmp = ff_get_filename_copy( );
 
         if ( ! fdopt.conv_only )
-            fl_show_alert2( 0, "Warning:\fFile %s\nwas created with an older "
-                            "fdesign version,\nthe new output file may not be "
-                            "compatible.", tmp );
+            fl_show_alert_f( 0, "Warning:\fFile %s\nwas created with an older "
+                             "fdesign version,\nthe new output file may not be "
+                             "compatible.", tmp );
         else
             M_warn( "", "Warning: File %s was created with an older fdesign "
                     "version, the new output file may not be compatible",
                     tmp );
 
-        fl_safe_free( tmp );
+        fli_safe_free( tmp );
     }
 
     if (    ff_read( "Internal Form Definition File" ) < 0
@@ -508,6 +543,24 @@ load_fd_header( void )
 
             set_snap_size( snap_size, 1 );
         }
+        else if ( ! strcmp( p, "Use-X11-Fonts" ) )
+        {
+            int state;
+
+
+            if ( ff_read( "%d", &state ) < 0 )
+                return ff_err( "Expected font type" );
+
+            state = state ? 1 : 0;
+#if FL_ENABLE_XFT
+            if ( state != fdopt.use_x11_fonts )
+            {
+                fl_set_default_font_type( state ? FL_X11_FONT : FL_XFT_FONT );
+                reverse_menu_marker( 5 );
+            }
+#endif
+            fdopt.use_x11_fonts = state;
+        }
         else if ( ! strcmp( p, "Border Width" ) )
         {
             int bw;
@@ -539,16 +592,14 @@ load_fd_header( void )
 
 int
 load_forms( int          merge,
-            const char * str,
-            int          record )
+            const char * str )
 {
     int i,
         saved_unit = fdopt.unit,
         r,
         nforms;
     FRM *new_forms;
-    char *fname,
-         *tmp;
+    char *fname;
 
     /* Try to open the .fd file */
 
@@ -595,12 +646,12 @@ load_forms( int          merge,
 
         if ( ! p || ! *p )
         {
-            fl_safe_free( p );
+            fli_safe_free( p );
             return ff_err( "Expected name of the form" );
         }
 
         fli_sstrcpy( forms[ fnumb ].fname, p, sizeof forms[ fnumb ].fname );
-        fl_safe_free( p );
+        fli_safe_free( p );
 
         /* Having gotten the name read all the remaining information. We then
            should either end up at the start of a new form or at the end of
@@ -637,20 +688,15 @@ load_forms( int          merge,
 
     set_form( fnumb > 0 ? 0 : -1 );
 
-    if ( merge )
-        changed = 1;
-    else if ( record && ( tmp = strrchr( fname, '/' ) ) )
-    {
-        fl_safe_free( loadedfile );
-        *tmp = '\0';
-        loadedfile = fl_strdup( fname );
-    }
+    fli_safe_free( loadedfile );
 
     if ( ! merge )
     {
-        fl_safe_free( loadedfile_fullpath );
-        loadedfile_fullpath = fl_strdup( fname );
+        loadedfile = rel2abs( fname );
+        changed = FL_FALSE;
     }
+    else
+        changed = FL_TRUE;
 
     fl_free( fname );
 
@@ -658,8 +704,8 @@ load_forms( int          merge,
 
     fli_cntl.coordUnit = FL_COORD_PIXEL;
 
-    /* Force output the same as input when converting directly. The reason is
-       we don't know the screen DPI. */
+    /* Force output to use the same unit as used in the input when converting
+       directly. The reason is that we don't know the screen DPI. */
 
     if ( ! fdopt.conv_only )
         fdopt.unit = saved_unit;
@@ -678,7 +724,7 @@ save_forms( const char *str )
 {
     int i,
         snap;
-    FILE *fn;
+    FILE *fp;
     char fname[ 1024 ],
          filename[ 1024 ];
     Conv *conv;
@@ -688,15 +734,28 @@ save_forms( const char *str )
     /* Get the filename if necessary */
 
     if ( ! str || ! * str )
-        str = fl_show_fselector( "Filename to save forms to", "",
-                                 "*.fd", loadedfile );
+    {
+        char *dir = NULL;
+
+        if ( loadedfile && strchr( loadedfile, '/' ) )
+        {
+            dir = fl_strdup( loadedfile );
+            *strrchr( dir, '/' ) = '\0';
+        }
+        else
+            dir = fl_strdup( "" );
+
+        str = fl_show_fselector( "Filename to save forms to", dir, "*.fd", "" );
+
+        fl_free( dir );
+    }
 
     if ( ! str )
         return 0;       /* cancel */
 
     if ( ! *str )
     {
-        fl_show_alert( "No forms were saved.", "", "", 0 );
+        fl_show_alert( "Warning", "No forms were saved.", "", 0 );
         return 0;
     }
 
@@ -704,47 +763,50 @@ save_forms( const char *str )
 
     strcpy( filename, str );
     i = strlen( filename ) - 1;
-    if ( strcmp( filename + i - 2, ".fd" ) == 0 )
+    if ( ! strcmp( filename + i - 2, ".fd" ) )
         filename[ i - 2 ] = '\0';
 
     strcpy( fname, filename );
 
-    /* On conversion there's need to (re)save  the .fd file. It messes up
-       Makefile rules */
+    /* In simple convert mode (i.e. not migrate) there's no need to (re)save
+       the .fd file */
 
-    if ( fdopt.conv_only )
+    if ( fdopt.conv_only == 1 )
         goto emit_code;
 
-    /* Make the .fd file. */
+    /* Make the .fd file */
 
     strcat( fname, ".fd" );
     make_backup( fname );
 
-    if ( ( fn = fopen( fname, "w" ) ) == 0 )
+    if ( ( fp = fopen( fname, "w" ) ) == 0 )
     {
-        fl_show_alert( "Cannot create definition file!!", "", "", 1 );
+        fl_show_alert_f( 1, "Error\fCannot open file\n%s\nfor writing",
+                         fname );
         return 0;
     }
 
     snap = get_step_size( ) + 0.1;
-    fprintf( fn, "Magic: %d\n\n", MAGIC5 );
-    fprintf( fn, "Internal Form Definition File\n" );
-    fprintf( fn, "    (do not change)\n\n" );
-    fprintf( fn, "Number of forms: %d\n", fnumb );
-
-    fprintf( fn, "Unit of measure: %s\n", unit_name( fdopt.unit ) );
+    fprintf( fp, "Magic: %d\n\n"
+                 "Internal Form Definition File\n"
+                 "    (do not change)\n\n"
+                 "Number of forms: %d\n"
+                 "Unit of measure: %s\n",
+             MAGIC7, fnumb, unit_name( fdopt.unit ) );
 
     if ( fd_bwidth != FL_BOUND_WIDTH && fd_bwidth )
-        fprintf( fn, "Border Width: %d\n", fd_bwidth );
+        fprintf( fp, "Border Width: %d\n", fd_bwidth );
 
     if ( snap != 10 )
-        fprintf( fn, "SnapGrid: %d\n", snap );
+        fprintf( fp, "SnapGrid: %d\n", snap );
+
+    fprintf( fp, "Use-X11-Fonts: %d\n", fdopt.use_x11_fonts );
 
     for ( i = 0; i < fnumb; i++ )
-        write_form( fn, forms[ i ].form, forms[ i ].fname );
+        write_form( fp, forms[ i ].form, forms[ i ].fname );
 
-    fprintf( fn, "\n==============================\n%s\n", main_name );
-    fclose( fn );
+    fprintf( fp, "\n==============================\n%s\n", main_name );
+    fclose( fp );
 
  emit_code:
 
@@ -755,7 +817,7 @@ save_forms( const char *str )
 
     conv = convertor + fdopt.language;
 
-    /* some converter works on the c code */
+    /* Some converter works on the c code */
 
     if ( conv->need_c_code )
         convertor[ FD_C ].convert( filename, forms, fnumb );
