@@ -409,6 +409,14 @@ static VN_pair vn_lstyle[ ] =
     VN( FL_FIXEDBOLD_STYLE ),
     VN( FL_FIXEDITALIC_STYLE ),
     VN( FL_FIXEDBOLDITALIC_STYLE ),
+    VN( FL_SERIF_STYLE ),
+    VN( FL_SERIFBOLD_STYLE ),
+    VN( FL_SERIFITALIC_STYLE ),
+    VN( FL_SERIFBOLDITALIC_STYLE ),
+    VN( FL_MISC_STYLE ),
+    VN( FL_MISCBOLD_STYLE ),
+    VN( FL_MISCITALIC_STYLE ),
+    VN( FL_MISCBOLDITALIC_STYLE ),
     VN( FL_TIMES_STYLE ),
     VN( FL_TIMESBOLD_STYLE ),
     VN( FL_TIMESITALIC_STYLE ),
@@ -883,13 +891,13 @@ get_conversion_factor( void )
     double sc = 1.0;
 
     if ( fdopt.unit == FL_COORD_POINT )
-        sc = 72.00 / fli_dpi;
+        sc = 72.00 / fl_dpi;
     else if ( fdopt.unit == FL_COORD_MM )
-        sc = 25.40 / fli_dpi;
+        sc = 25.40 / fl_dpi;
     else if ( fdopt.unit == FL_COORD_centiPOINT )
-        sc = 7200.00 / fli_dpi;
+        sc = 7200.00 / fl_dpi;
     else if ( fdopt.unit == FL_COORD_centiMM )
-        sc = 2540.00 / fli_dpi;
+        sc = 2540.00 / fl_dpi;
 
     return sc;
 }
@@ -1136,8 +1144,8 @@ get_free_handle( FL_OBJECT  * ob,
     static char buf[ 1024 ];
     static FL_OBJECT *freeobj[ MAXFREEOBJ ];
 
-    if ( ob->c_vdata )
-        strcpy( buf, ob->c_vdata );
+    if ( ob->fdesign_vdata )
+        strcpy( buf, ob->fdesign_vdata );
     else if ( *name )
         sprintf( buf, "freeobj_%s_handler", name );
     else if ( *ob->label )
@@ -1412,6 +1420,11 @@ output_main_newformat( FILE * fn,
 
     fprintf( fn, "    fl_initialize( &argc, argv, 0, 0, 0 );\n" );
 
+    if ( fdopt.use_x11_fonts )
+        fprintf( fn, "#if FL_ENABLE_XFT\n"
+                     "    fl_set_default_font_type( FL_X11_FONT );\n"
+                     "#endif\n\n" );
+
     for ( i = 0; i < nform; i++ )
     {
         fname = fdform[ i ].fname;
@@ -1455,6 +1468,10 @@ output_main_altformat( FILE * fn,
     fprintf( fn, "int\nmain( int    argc,\n      char * argv[ ] )\n{\n" );
 
     fprintf( fn, "    fl_initialize( &argc, argv, 0, 0, 0 );\n\n" );
+    if ( fdopt.use_x11_fonts )
+        fprintf( fn, "#if FL_ENABLE_XFT\n"
+                     "    fl_set_default_font+type( FL_X11_FONT_TYPE );\n"
+                     "#endif\n\n" );
     fprintf( fn, "\n     %s( );\n\n", main_name );
     fprintf( fn, "\n     /* Fill-in form initialization code */\n\n" );
     fprintf( fn, "    /* Show the first form */\n\n" );
@@ -1635,6 +1652,25 @@ output_object( FILE      * fp,
             fprintf( fp, "    fl_set_object_lstyle( obj, %s );\n",
                      style_name( obj->lstyle ) );
 
+        if ( obj->react_to != defobj->react_to )
+        {
+            if ( ! obj->react_to )
+                fprintf( fp, "    fl_set_object_mouse_buttons( obj, 0 );\n" );
+            else
+            {
+                int i;
+                int first = 0;
+
+                fprintf( fp, "    fl_set_object_mouse_buttons( obj, " );
+                for ( i = FL_MBUTTON1; i <= FL_MBUTTON5; i++ )
+                    if ( REACT_TO( obj, i ) )
+                        fprintf( fp, "%sFL_MBUTTON%d_BIT",
+                                 first++ ? " | " : "", i );
+
+                fprintf( fp, " );\n" );
+            }
+        }
+
         /* 'resize' must be checked for consistency with the gravity settings */
 
         obj->resize = check_resize( obj->resize,
