@@ -27,17 +27,21 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "include/forms.h"
 #include "flinternal.h"
 #include "private/pchoice.h"
-#include "private/flsnprintf.h"
+#include "private/flvasprintf.h"
 #include <string.h>
 #include <stdlib.h>
 
 
+#define NO_TITLE_MASK      1
+#define ALIGN_BOTTOM_MASK  2
+
+
 
 /***************************************
  ***************************************/
@@ -50,9 +54,9 @@ free_choice( FLI_CHOICE_SPEC * sp )
     for ( i = 1; i <= FL_CHOICE_MAXITEMS; i++ )
     {
         if ( sp->items[ i ] )
-            fl_safe_free( sp->items[ i ] );
+            fli_safe_free( sp->items[ i ] );
         if ( sp->shortcut[ i ] )
-            fl_safe_free( sp->shortcut[ i ] );
+            fli_safe_free( sp->shortcut[ i ] );
     }
 }
 
@@ -70,29 +74,30 @@ draw_choice( FL_OBJECT * ob )
     int off1 = 0,
         off2 = 0;
 
-    c1 = ob->belowmouse ? FL_CHOICE_MCOL : ob->col1;
+    c1 = ob->belowmouse ? FLI_CHOICE_MCOL : ob->col1;
 
-    fl_drw_box( ob->boxtype, ob->x, ob->y, ob->w, ob->h, c1, ob->bw );
+    fl_draw_box( ob->boxtype, ob->x, ob->y, ob->w, ob->h, c1, ob->bw );
 
     if ( ob->type == FL_NORMAL_CHOICE2 )
     {
         int dh = FL_max( 6 + ( ob->bw > 0 ), ob->h * 0.1 );
         int dw = FL_max( 0.11 * ob->w, 13 );
         int dbh = FL_max( absbw - 1, 1 );
-        int align = sp->align & ~ FL_ALIGN_INSIDE;
+        int align = fl_to_outside_lalign( sp->align );
 
-        fl_drw_box( FL_UP_BOX,
-                    ob->x + ob->w - dw - absbw - 2, ob->y + ( ob->h - dh ) / 2,
-                    dw, dh, ob->col1, - dbh );
+        fl_draw_box( FL_UP_BOX,
+                     ob->x + ob->w - dw - absbw - 2,
+                     ob->y + ( ob->h - dh ) / 2,
+                     dw, dh, ob->col1, - dbh );
 
-        off1 = align == FL_ALIGN_CENTER ? ( dw / 2 ) : 0;
+        off1 = fl_is_center_lalign( align ) ? ( dw / 2 ) : 0;
         off2 = align == FL_ALIGN_RIGHT ? dw : 0;
     }
 
-    fl_drw_text_beside( ob->align, ob->x, ob->y, ob->w, ob->h, ob->lcol,
-                        ob->lstyle, ob->lsize, ob->label );
+    fl_draw_text_beside( ob->align, ob->x, ob->y, ob->w, ob->h, ob->lcol,
+                         ob->lstyle, ob->lsize, ob->label );
 
-    /* string can conceivably contain "type flags". need to get rid of them
+    /* String can conceivably contain "type flags". need to get rid of them
        on the fly */
 
     if ( sp->val > 0 && sp->val <= sp->numitems )
@@ -110,9 +115,9 @@ draw_choice( FL_OBJECT * ob )
 
         fl_set_text_clipping( ob->x + absbw, ob->y,
                               ob->w - 2 * absbw, ob->h );
-        fl_drw_text( sp->align, ob->x - off1, ob->y, ob->w - off2,
-                     ob->h, ob->col2, sp->fontstyle, sp->fontsize,
-                     str + ( str && ( *str == '\010' ) ) );
+        fl_draw_text( sp->align, ob->x - off1, ob->y, ob->w - off2,
+                      ob->h, ob->col2, sp->fontstyle, sp->fontsize,
+                      str + ( str && ( *str == '\010' ) ) );
         fl_unset_text_clipping( );
         fl_free( str );
     }
@@ -131,25 +136,31 @@ draw_droplist_choice( FL_OBJECT * ob )
              dx = ob->w - dw,
              bw;
 
-    c1 = sp->below ? FL_CHOICE_MCOL : ob->col1;
+    c1 = sp->below ? FLI_CHOICE_MCOL : ob->col1;
     bw = ob->bw;
     if ( bw > 0 )
         bw -= ob->bw > 1;
 
-    /* arrows */
+    /* Arrows */
 
-    fl_drw_box( sp->pushed ? FL_DOWN_BOX : FL_UP_BOX, ob->x + dx, ob->y,
-                dw, ob->h, c1, bw );
-    fl_drw_text( FL_ALIGN_CENTER, ob->x + dx + 2, ob->y + 2, dw - 4, ob->h - 4,
-                 FL_BLACK, 0, 0, "@#2->" );
+    fl_draw_box( sp->pushed ? FL_DOWN_BOX : FL_UP_BOX, ob->x + dx, ob->y,
+                 dw, ob->h, c1, bw );
+    if ( sp->no_title & ALIGN_BOTTOM_MASK )
+        fl_draw_text( FL_ALIGN_CENTER, ob->x + dx + 2, ob->y + 2,
+                      dw - 4, ob->h - 4,
+                      FL_BLACK, 0, 0, "@#8->" );
+    else
+        fl_draw_text( FL_ALIGN_CENTER, ob->x + dx + 2, ob->y + 2,
+                      dw - 4, ob->h - 4,
+                      FL_BLACK, 0, 0, "@#2->" );
 
-    /* choice box */
+    /* Choice box */
 
-    fl_drw_box( ob->boxtype, ob->x, ob->y, dx, ob->h, ob->col1, ob->bw );
-    fl_drw_text_beside( ob->align, ob->x, ob->y, dx, ob->h, ob->lcol,
-                        ob->lstyle, ob->lsize, ob->label );
+    fl_draw_box( ob->boxtype, ob->x, ob->y, dx, ob->h, ob->col1, ob->bw );
+    fl_draw_text_beside( ob->align, ob->x, ob->y, dx, ob->h, ob->lcol,
+                         ob->lstyle, ob->lsize, ob->label );
 
-    /* string can conceivably contain "type flags", need to get rid of them
+    /* String can conceivably contain "type flags", need to get rid of them
        on the fly */
 
     if ( sp->val > 0 && sp->val <= sp->numitems )
@@ -167,9 +178,9 @@ draw_droplist_choice( FL_OBJECT * ob )
 
         fl_set_text_clipping( ob->x + FL_abs( ob->bw ), ob->y,
                               ob->w - 2 * FL_abs( ob->bw ), ob->h );
-        fl_drw_text( sp->align, ob->x, ob->y, dx, ob->h, ob->col2,
-                     sp->fontstyle, sp->fontsize,
-                     str + ( str && ( *str == '\010' ) ) );
+        fl_draw_text( sp->align, ob->x, ob->y, dx, ob->h, ob->col2,
+                      sp->fontstyle, sp->fontsize,
+                      str + ( str && ( *str == '\010' ) ) );
         fl_unset_text_clipping( );
         fl_free( str );
     }
@@ -235,12 +246,12 @@ do_pup( FL_OBJECT * ob )
 
     popup_id = fl_newpup( FL_ObjWin( ob ) );
 
-    /* fake a title */
+    /* Fake a title */
 
     if (    ob->label
          && ob->label[ 0 ]
          && ob->type != FL_DROPLIST_CHOICE
-         && ! sp->no_title )
+         && ! ( sp->no_title & NO_TITLE_MASK ) )
     {
         char *t = fl_malloc( strlen( ob->label ) + 3 );
 
@@ -268,6 +279,9 @@ do_pup( FL_OBJECT * ob )
 
     fl_setpup_softedge( popup_id, ob->bw < 0 );
 
+    if ( sp->no_title & ALIGN_BOTTOM_MASK )
+        fl_setpup_align_bottom( );
+
     val = fl_dopup( popup_id );
 
     if ( val > 0 )
@@ -310,13 +324,13 @@ handle_choice( FL_OBJECT * ob,
 
     switch ( event )
     {
-        case FL_DRAW:
+        case FL_ATTRIB :
             /* always force outside alignment */
 
-            ob->align &= ~FL_ALIGN_INSIDE;
-
-            /* Draw the object */
+            ob->align = fl_to_outside_lalign( ob->align );
+            break;
 
+        case FL_DRAW:
             if ( ob->type == FL_DROPLIST_CHOICE )
                 draw_droplist_choice( ob );
             else
@@ -324,11 +338,14 @@ handle_choice( FL_OBJECT * ob,
             break;
 
         case FL_DRAWLABEL:
-            fl_drw_text_beside( ob->align, ob->x, ob->y, ob->w, ob->h, ob->lcol,
-                                ob->lstyle, ob->lsize, ob->label );
+            fl_draw_text_beside( ob->align, ob->x, ob->y, ob->w, ob->h,
+                                 ob->lcol, ob->lstyle, ob->lsize, ob->label );
             break;
 
         case FL_PUSH:
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             if ( key == FL_MBUTTON2 || key == FL_MBUTTON3 )
             {
                 sp->counter = 0;
@@ -361,6 +378,9 @@ handle_choice( FL_OBJECT * ob,
             break;
 
         case FL_UPDATE:
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             if (    ( key == FL_MBUTTON2 || key == FL_MBUTTON3 )
                  && ++sp->counter % 15 == 0 )
             {
@@ -374,6 +394,9 @@ handle_choice( FL_OBJECT * ob,
             break;
 
         case FL_MOTION:
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             if ( sp->numitems == 0 || ob->type != FL_DROPLIST_CHOICE )
                 break;
 
@@ -393,6 +416,9 @@ handle_choice( FL_OBJECT * ob,
             break;
                 
         case FL_RELEASE:
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             if ( sp->numitems == 0 )
                 break;
 
@@ -417,8 +443,12 @@ handle_choice( FL_OBJECT * ob,
                 break;
             }
 
-            fl_setpup_position( - ( ob->form->x + ob->x + ob->w ),
-                                ob->form->y + ob->y + ob->h + FL_PUP_PADH );
+            if ( sp->no_title & ALIGN_BOTTOM_MASK )
+                fl_setpup_position( - ( ob->form->x + ob->x + ob->w ),
+                                    ob->form->y + ob->y - FL_PUP_PADH );
+            else
+                fl_setpup_position( - ( ob->form->x + ob->x + ob->w ),
+                                    ob->form->y + ob->y + ob->h + FL_PUP_PADH );
             sp->pushed = 0;
             if ( do_pup( ob ) > 0 )
                 ret |= FL_RETURN_CHANGED | FL_RETURN_END;
@@ -465,7 +495,7 @@ handle_choice( FL_OBJECT * ob,
 
 
 /***************************************
- * creates an object
+ * Creates an object
  ***************************************/
 
 FL_OBJECT *
@@ -482,11 +512,12 @@ fl_create_choice( int          type,
 
     obj = fl_make_object( FL_CHOICE, type, x, y, w, h, label, handle_choice );
 
-    obj->boxtype     = type == FL_NORMAL_CHOICE2 ? FL_UP_BOX : FL_CHOICE_BOXTYPE;
-    obj->col1        = FL_CHOICE_COL1;
-    obj->col2        = FL_CHOICE_COL2;
-    obj->lcol        = FL_CHOICE_LCOL;
-    obj->align       = FL_CHOICE_ALIGN;
+    obj->boxtype     = type == FL_NORMAL_CHOICE2 ?
+                               FL_UP_BOX : FLI_CHOICE_BOXTYPE;
+    obj->col1        = FLI_CHOICE_COL1;
+    obj->col2        = FLI_CHOICE_COL2;
+    obj->lcol        = FLI_CHOICE_LCOL;
+    obj->align       = FLI_CHOICE_ALIGN;
     obj->want_update = 1;
     obj->spec = sp   = fl_calloc( 1, sizeof *sp );
 
@@ -554,7 +585,7 @@ fl_clear_choice( FL_OBJECT * ob )
 
 
 /***************************************
- * add a single choice item
+ * Add a single choice item
  ***************************************/
 
 static void
@@ -582,7 +613,7 @@ addto_choice( FL_OBJECT *  ob,
 
 
 /***************************************
- * user interface routine. | allowed
+ * User interface routine.
  ***************************************/
 
 int
@@ -621,6 +652,24 @@ fl_addto_choice( FL_OBJECT *  ob,
 }
 
 
+/***************************************
+ ***************************************/
+
+int
+fl_addto_choice_f( FL_OBJECT *  ob,
+                   const char * fmt,
+                   ... )
+{
+    int n;
+    char *buf;
+
+    EXPAND_FORMAT_STRING( buf, fmt );
+    n = fl_addto_choice( ob, buf );
+    fl_free( buf );
+    return n;
+}
+
+
 /***************************************
  * Replaces a line to the choice item.
  ***************************************/
@@ -644,6 +693,23 @@ fl_replace_choice( FL_OBJECT *  ob,
 }
 
 
+/***************************************
+ ***************************************/
+
+void
+fl_replace_choice_f( FL_OBJECT *  ob,
+                     int          numb,
+                     const char * fmt,
+                     ... )
+{
+    char *buf;
+
+    EXPAND_FORMAT_STRING( buf, fmt );
+    fl_replace_choice( ob, numb, buf );
+    fl_free( buf );
+}
+
+
 /***************************************
  * Removes a line from the choice item.
  ***************************************/
@@ -705,7 +771,7 @@ fl_set_choice( FL_OBJECT * ob,
 
 
 /***************************************
- * similar to set_choice, except we use txt
+ * Similar to set_choice, except we use txt
  ***************************************/
 
 void
@@ -736,6 +802,22 @@ fl_set_choice_text( FL_OBJECT *  ob,
 }
 
 
+/***************************************
+ ***************************************/
+
+void
+fl_set_choice_text_f( FL_OBJECT *  ob,
+                      const char * fmt,
+                      ... )
+{
+    char *buf;
+
+    EXPAND_FORMAT_STRING( buf, fmt );
+    fl_set_choice_text( ob, buf );
+    fl_free( buf );
+}
+
+
 /***************************************
  ***************************************/
 
@@ -796,6 +878,7 @@ fl_set_choice_item_shortcut( FL_OBJECT *  ob,
 
     if ( sp->shortcut[ item ] )
         fl_free( sp->shortcut[ item ] );
+
     sp->shortcut[ item ] = fl_strdup( sc ? sc : "" );
 }
 
@@ -879,6 +962,12 @@ fl_set_choice_fontsize( FL_OBJECT * ob,
 {
     FLI_CHOICE_SPEC *sp = ob->spec;
 
+    if ( size < 1 )
+    {
+        M_err( "fl_set_choice_fontsize", "Invalid font size (%d)", size );
+        return;
+    }
+
     if ( sp->fontsize != size )
     {
         sp->fontsize = size;
@@ -954,13 +1043,38 @@ fl_set_choice_notitle( FL_OBJECT * ob,
                        int         n )
 {
     FLI_CHOICE_SPEC *sp = ob->spec;
-    int old = sp->no_title;
+    int old = sp->no_title & NO_TITLE_MASK ? 1 : 0;
+
+    if ( n )
+        sp->no_title |= NO_TITLE_MASK;
+    else
+        sp->no_title &= ~ NO_TITLE_MASK;
 
-    sp->no_title = n;
     return old;
 }
 
 
+/***************************************
+ ***************************************/
+
+int
+fl_set_choice_align_bottom( FL_OBJECT * ob,
+                            int         n )
+{
+    FLI_CHOICE_SPEC *sp = ob->spec;
+    int old = sp->no_title & ALIGN_BOTTOM_MASK ? 1 : 0;
+
+    if ( n )
+        sp->no_title |= ALIGN_BOTTOM_MASK;
+    else
+        sp->no_title &= ~ ALIGN_BOTTOM_MASK;
+
+    return old;
+}
+
+
+
+
 /*
  * Local variables:
  * tab-width: 4
