@@ -30,6 +30,7 @@
 
 #include "include/forms.h"
 #include "flinternal.h"
+#include "private/pclock.h"
 
 #include <math.h>
 #include <time.h>
@@ -207,8 +208,8 @@ draw_digitalclock( FL_OBJECT * ob )
     else
         sprintf( buf, "%d:%02d:%02d", hours, minutes, seconds );
 
-    fl_drw_text( FL_ALIGN_CENTER, ob->x, ob->y, ob->w, ob->h, ob->col2,
-                 ob->lstyle, ob->lsize, buf );
+    fl_draw_text( FL_ALIGN_CENTER, ob->x, ob->y, ob->w, ob->h, ob->col2,
+                  ob->lstyle, ob->lsize, buf );
 }
 
 
@@ -235,8 +236,8 @@ handle_clock( FL_OBJECT * ob,
             break;
 
         case FL_DRAW :
-            fl_drw_box( ob->boxtype, ob->x, ob->y, ob->w, ob->h,
-                        ob->col1, ob->bw );
+            fl_draw_box( ob->boxtype, ob->x, ob->y, ob->w, ob->h,
+                         ob->col1, ob->bw );
             if ( ob->type == FL_DIGITAL_CLOCK )
                 draw_digitalclock( ob );
             else
@@ -246,10 +247,10 @@ handle_clock( FL_OBJECT * ob,
 
         case FL_DRAWLABEL :
             if ( ! updating )
-                fl_drw_text_beside( ob->align & ~ FL_ALIGN_INSIDE,
-                                    ob->x, ob->y, ob->w, ob->h,
-                                    ob->lcol, ob->lstyle, ob->lsize,
-                                    ob->label );
+                fl_draw_text_beside( ob->align & ~ FL_ALIGN_INSIDE,
+                                     ob->x, ob->y, ob->w, ob->h,
+                                     ob->lcol, ob->lstyle, ob->lsize,
+                                     ob->label );
             updating = 0;
             break;
 
@@ -301,11 +302,11 @@ fl_create_clock( int          type,
 
     obj = fl_make_object( FL_CLOCK, type, x, y, w, h, s, handle_clock );
 
-    obj->boxtype   = FL_CLOCK_BOXTYPE;
-    obj->col1      = FL_CLOCK_COL1;
-    obj->col2      = FL_CLOCK_COL2;
-    obj->lcol      = FL_CLOCK_LCOL;
-    obj->align     = FL_CLOCK_ALIGN;
+    obj->boxtype   = FLI_CLOCK_BOXTYPE;
+    obj->col1      = FLI_CLOCK_COL1;
+    obj->col2      = FLI_CLOCK_COL2;
+    obj->lcol      = FLI_CLOCK_LCOL;
+    obj->align     = FLI_CLOCK_ALIGN;
     obj->automatic = obj->active = 1;
     obj->spec = sp = fl_calloc( 1, sizeof *sp );
 
