@@ -117,44 +117,44 @@ draw_counter( FL_OBJECT * ob )
 
     if ( ob->type == FL_NORMAL_COUNTER && sp->draw_type & OB0 )
     {
-        fl_drw_box( btype[ 0 ], sp->xx[ 0 ], ob->y, sp->ww[ 0 ], ob->h,
-                    ob->col1, ob->bw );
-        fl_drw_text( FL_ALIGN_CENTER, sp->xx[ 0 ], ob->y, sp->ww[ 0 ], ob->h,
-                     ob->col2, 0, 0, "@#<<" );
+        fl_draw_box( btype[ 0 ], sp->xx[ 0 ], ob->y, sp->ww[ 0 ], ob->h,
+                     ob->col1, ob->bw );
+        fl_draw_text( FL_ALIGN_CENTER, sp->xx[ 0 ], ob->y, sp->ww[ 0 ], ob->h,
+                      ob->col2, 0, 0, "@#<<" );
     }
 
     if ( sp->draw_type & OB1 )
     {
-        fl_drw_box( btype[ 1 ], sp->xx[ 1 ], ob->y, sp->ww[ 1 ], ob->h,
-                    ob->col1, ob->bw );
-        fl_drw_text( FL_ALIGN_CENTER, sp->xx[ 1 ], ob->y, sp->ww[ 1 ], ob->h,
-                     ob->col2, 0, 0, "@#<" );
+        fl_draw_box( btype[ 1 ], sp->xx[ 1 ], ob->y, sp->ww[ 1 ], ob->h,
+                     ob->col1, ob->bw );
+        fl_draw_text( FL_ALIGN_CENTER, sp->xx[ 1 ], ob->y, sp->ww[ 1 ], ob->h,
+                      ob->col2, 0, 0, "@#<" );
     }
 
     if ( sp->draw_type & OB4 )
     {
-        fl_drw_box( btype[ 4 ], sp->xx[ 4 ], ob->y, sp->ww[ 4 ], ob->h,
-                    ob->col1, ob->bw );
+        fl_draw_box( btype[ 4 ], sp->xx[ 4 ], ob->y, sp->ww[ 4 ], ob->h,
+                     ob->col1, ob->bw );
         fl_set_text_clipping( sp->xx[ 4 ], ob->y, sp->ww[ 4 ], ob->h );
-        fl_drw_text( FL_ALIGN_CENTER, sp->xx[ 4 ], ob->y, sp->ww[ 4 ], ob->h,
-                     ob->lcol, ob->lstyle, ob->lsize, str );
+        fl_draw_text( FL_ALIGN_CENTER, sp->xx[ 4 ], ob->y, sp->ww[ 4 ], ob->h,
+                      ob->lcol, ob->lstyle, ob->lsize, str );
         fl_unset_text_clipping( );
     }
 
     if ( sp->draw_type & OB2 )
     {
-        fl_drw_box( btype[ 2 ], sp->xx[ 2 ], ob->y, sp->ww[ 2 ], ob->h,
-                    ob->col1, ob->bw );
-        fl_drw_text( FL_ALIGN_CENTER, sp->xx[ 2 ], ob->y, sp->ww[ 2 ], ob->h,
-                     ob->col2, 0, 0, "@#>" );
+        fl_draw_box( btype[ 2 ], sp->xx[ 2 ], ob->y, sp->ww[ 2 ], ob->h,
+                     ob->col1, ob->bw );
+        fl_draw_text( FL_ALIGN_CENTER, sp->xx[ 2 ], ob->y, sp->ww[ 2 ], ob->h,
+                      ob->col2, 0, 0, "@#>" );
     }
 
     if ( ob->type == FL_NORMAL_COUNTER && sp->draw_type & OB3 )
     {
-        fl_drw_box( btype[ 3 ], sp->xx[ 3 ], ob->y, sp->ww[ 3 ], ob->h,
-                    ob->col1, ob->bw );
-        fl_drw_text( FL_ALIGN_CENTER, sp->xx[ 3 ], ob->y, sp->ww[ 3 ], ob->h,
-                     ob->col2, 0, 0, "@#>>" );
+        fl_draw_box( btype[ 3 ], sp->xx[ 3 ], ob->y, sp->ww[ 3 ], ob->h,
+                     ob->col1, ob->bw );
+        fl_draw_text( FL_ALIGN_CENTER, sp->xx[ 3 ], ob->y, sp->ww[ 3 ], ob->h,
+                      ob->col2, 0, 0, "@#>>" );
     }
 
     if ( sp->draw_type == ALL )
@@ -467,7 +467,7 @@ handle_counter( FL_OBJECT * ob,
 
         case FL_PUSH:
         case FL_UPDATE:
-            if ( key != FL_MBUTTON1 )
+            if ( ! REACT_TO( ob, key ) )
                 break;
 
             if (    ( ret |= handle_mouse( ob, event, mx, my ) )
@@ -476,7 +476,7 @@ handle_counter( FL_OBJECT * ob,
             break;
 
         case FL_RELEASE:
-            if ( key != FL_MBUTTON1 )
+            if ( ! REACT_TO( ob, key ) )
                 break;
 
             ret |= handle_mouse( ob, event, mx, my );
@@ -486,6 +486,10 @@ handle_counter( FL_OBJECT * ob,
             break;
 
         case FL_MOTION:
+            if ( key != FL_MBUTTON1 )
+                break;
+            /* fallthrough */
+
         case FL_ENTER:
         case FL_LEAVE:
             show_focus_obj( ob, mx, my );
@@ -516,14 +520,13 @@ fl_create_counter( int          type,
     FLI_COUNTER_SPEC *sp;
 
     ob = fl_make_object( FL_COUNTER, type, x, y, w, h, label, handle_counter );
-    ob->boxtype     = FL_COUNTER_BOXTYPE;
-    ob->col1        = FL_COUNTER_COL1;
-    ob->col2        = FL_COUNTER_COL2;
-    ob->align       = FL_COUNTER_ALIGN;
-    ob->lcol        = FL_COUNTER_LCOL;
+    ob->boxtype     = FLI_COUNTER_BOXTYPE;
+    ob->col1        = FLI_COUNTER_COL1;
+    ob->col2        = FLI_COUNTER_COL2;
+    ob->align       = FLI_COUNTER_ALIGN;
+    ob->lcol        = FLI_COUNTER_LCOL;
     ob->want_motion = 1;
     ob->want_update = 1;
-//    ob->bw          = FL_COUNTER_BW;
 
     sp = ob->spec     = fl_calloc( 1, sizeof *sp );
     sp->min           = -1000000.0;
@@ -544,6 +547,8 @@ fl_create_counter( int          type,
 
     fl_set_object_return( ob, FL_RETURN_CHANGED );
 
+    fl_set_object_mouse_buttons( ob, FL_MBUTTON1_BIT );
+
     return ob;
 }
 
