@@ -87,6 +87,10 @@
 #endif
 
 
+#define REACT_TO( obj_, key_ ) \
+   ( ( ( obj_ )->react_to & ( 1U << ( ( key_ ) - 1 ) ) ) ? 1 : 0 )
+
+
 /* If we want to have gamma correction as a built-in feature.
  * Probably useless */
 
@@ -278,7 +282,7 @@ Window fli_create_window( Window,
                           Colormap,
                           const char * );
 
-void fli_create_gc( Window );
+void fli_create_gc( Drawable );
 
 enum {
     FLI_COMMAND_PROP = 1,
@@ -292,6 +296,9 @@ void fli_set_winproperty( Window,
 
 void fli_init_colormap( int );
 
+int fli_get_text_width( const char *,
+                        size_t  );
+
 void fli_free_colormap( int );
 
 void fli_dump_state_info( int,
@@ -299,8 +306,6 @@ void fli_dump_state_info( int,
 
 void fli_init_stipples( void );
 
-void fli_draw_button( FL_OBJECT * );
-
 /* for fdesign */
 
 const char * fli_query_colorname( FL_COLOR );
@@ -323,43 +328,39 @@ int fli_get_pos_in_string( int,
                            int *,
                            int * );
 
-int fli_drw_stringTAB( Window,
-                       GC,
-                       int,
-                       int,
-                       int,
-                       int,
-                       const char *,
-                       int,
-                       int );
+int fli_draw_stringTAB( Drawable,
+                        FL_COLOR,
+                        int,
+                        int,
+                        int,
+                        int,
+                        const char *,
+                        int,
+                        int );
 
-int fli_drw_string( int,
-                    FL_Coord,
-                    FL_Coord,
-                    FL_Coord,
-                    FL_Coord,
-                    int,
-                    FL_COLOR,
-                    FL_COLOR,
-                    FL_COLOR,
-                    int,
-                    int,
-                    int,
-                    int,
-                    int,
-                    const char *,
-                    int,
-                    int,
-                    int,
-                    FL_COLOR );
+int fli_draw_string( int,
+                     FL_Coord,
+                     FL_Coord,
+                     FL_Coord,
+                     FL_Coord,
+                     int,
+                     FL_COLOR,
+                     FL_COLOR,
+                     FL_COLOR,
+                     int,
+                     int,
+                     int,
+                     int,
+                     int,
+                     const char *,
+                     int,
+                     int,
+                     int,
+                     FL_COLOR );
 
 int fli_get_max_pixels_line( void );
 
-int fli_get_string_widthTABfs( XFontStruct *,
-                               const char *,
-                               int );
-
-void fli_init_font( void );
+void fli_init_fonts( void );
 
 void fli_canonicalize_rect( FL_Coord *,
                             FL_Coord *,
@@ -384,7 +385,12 @@ void fli_check_key_focus( const char *,
 
 void fli_free_cmdline_args( void );
 
-FL_RECT * fli_get_underline_rect( XFontStruct *,
+FL_RECT * fli_get_underline_rect(
+#if FL_ENABLE_XFT
+                                  XftFont     *,
+#else
+                                  XFontStruct *,
+#endif
                                   FL_Coord,
                                   FL_Coord,
                                   const char *,
@@ -418,19 +424,19 @@ enum {
 void fli_calc_slider_size( FL_OBJECT *,
                            FLI_SCROLLBAR_KNOB * );
 
-void fli_drw_slider( FL_OBJECT *,
-                     FL_COLOR,
-                     FL_COLOR,
-                     const char *,
-                     int );
+void fli_draw_slider( FL_OBJECT *,
+                      FL_COLOR,
+                      FL_COLOR,
+                      const char *,
+                      int );
 
-void fli_drw_checkbox( int      type,
-                       FL_Coord x,
-                       FL_Coord y,
-                       FL_Coord w,
-                       FL_Coord h,
-                       FL_COLOR col,
-                       int      bw );
+void fli_draw_checkbox( int      type,
+                        FL_Coord x,
+                        FL_Coord y,
+                        FL_Coord w,
+                        FL_Coord h,
+                        FL_COLOR col,
+                        int      bw );
 
 void fli_set_global_clipping( FL_Coord,
                               FL_Coord,
@@ -447,6 +453,9 @@ void fli_set_additional_clipping( FL_Coord,
 FL_RECT * fli_get_global_clip_rect( void );
 
 
+void fli_apply_clipping_to_gc( GC gc );
+
+
 /* Application windows */
 
 typedef struct fli_win_ {
@@ -462,8 +471,6 @@ typedef struct fli_win_ {
 
 extern FLI_WIN * fl_app_win;
 
-void fli_set_form_window( FL_FORM * );
-
 void fli_unmap_canvas_window( FL_OBJECT * );
 
 FL_APPEVENT_CB fli_set_preemptive_callback( Window,
@@ -553,7 +560,7 @@ typedef struct fli_context_ {
                          vscb;              /* default scrollbar      */
     long                 ext_request_size;  /* extended request size  */
     int                  tooltip_time;
-#ifdef XlibSpecificationRelease
+#if defined XlibSpecificationRelease
     XIM                  xim;               /* input method           */
     XIC                  xic;               /* input context          */
 #else
@@ -570,20 +577,28 @@ typedef struct fli_context_ {
 typedef struct {
     Display       * display;
     Window          win;
-    GC              gc,
-                    textgc;
-    GC              miscgc;
+    GC              gc;
+#if FL_ENABLE_XFT
+    XftDraw       * textdraw;
+    XftDraw       * bgdraw;
+#else
+    GC              textgc;
+#endif
     int             isRGBColor;
-    int             isMBFont;       /* multi-byte font       */
-    unsigned long   bktextcolor;
     int             newpix;
     int             fdesc;          /* font descent          */
     int             fasc;           /* font ascent           */
     int             fheight;        /* font height           */
-    Colormap        colormap;
+#if FL_ENABLE_XFT
+    XftFont       * fs;
+    XftColor        textcolor;
+    XftColor        bktextcolor;
+#else
     XFontStruct   * fs;
-    unsigned long   color;          /* last color. cache     */
     unsigned long   textcolor;      /* last textcolor. cache */
+    unsigned long   bktextcolor;
+#endif
+    unsigned long   color;          /* last color. cache     */
     unsigned long   bkcolor;
     int             screen;
 } FLI_TARGET;
@@ -774,7 +789,11 @@ int fli_get_visible_forms_index( FL_FORM * );
 
 void fli_recount_auto_objects( void );
 
+#if FL_ENABLE_XFT
+int fli_get_tabpixels( XftFont * );
+#else
 int fli_get_tabpixels( XFontStruct * );
+#endif
 
 int fli_get_default_scrollbarsize( FL_OBJECT * );
 
@@ -894,9 +913,6 @@ char * fli_sstrcpy( char       * dest,
                     const char * src,
                     size_t       n );
 
-void fli_set_form_icon_data( FL_FORM *,
-                             char ** );
-
 char *fli_getcwd( char *,
                   int );
 
@@ -1100,11 +1116,51 @@ void fli_set_ul_property( int prop,
 
 int fli_is_valid_dir( const char * name );
 
-void fli_color_chooser_cleanup( void );
+#if FL_ENABLE_XFT
+int fli_font_index_compare( const void * arg1,
+                            const void * arg2 );
+
+char * fli_convert_to_full_xlfd( const char * pattern );
+
+char * fli_get_fname( const FL_FONT * font,
+                      int             size );
+
+int fli_get_string_width( XftFont    * f,
+                          const char * s,
+                          int          len );
+
+int fli_get_string_widthTABfs( XftFont    * fs,
+                               const char * s,
+                               int          len );
+
+XftFont * fli_get_font_struct( int style,
+                               int size );
+
+char * fli_cv_fname( const FL_FONT * f );
+
+unsigned long fli_get_xrendercolor( FL_COLOR       i,
+                                    XRenderColor * xrd );
+#else
+int fli_get_string_width( XFontStruct * f,
+                          const char  * s,
+                          int           len );
+
+int fli_get_string_widthTABfs( XFontStruct *,
+                               const char *,
+                               int );
+#endif
+
+
+void * fli_remove_and_resize( void   * arr,
+                              size_t   memb_size,
+                              size_t * len,
+                              size_t   index,
+                              size_t   remove_cnt );
 
 #endif /* ! defined FL_INTERNAL_H */
 
 
+
 /*
  * Local variables:
  * tab-width: 4
