@@ -30,6 +30,7 @@
 
 #include "include/forms.h"
 #include "flinternal.h"
+#include "private/pframe.h"
 
 
 /***************************************
@@ -50,13 +51,13 @@ handle_frame( FL_OBJECT * ob,
             break;
 
         case FL_DRAW:
-            fl_drw_frame( ob->type, ob->x, ob->y, ob->w, ob->h,
-                          ob->col1, ob->bw );
+            fl_draw_frame( ob->type, ob->x, ob->y, ob->w, ob->h,
+                           ob->col1, ob->bw );
             /* fall through */
 
         case FL_DRAWLABEL:
-            fl_drw_text( ob->align, ob->x, ob->y, ob->w, ob->h,
-                         ob->lcol, ob->lstyle, ob->lsize, ob->label );
+            fl_draw_text( ob->align, ob->x, ob->y, ob->w, ob->h,
+                          ob->lcol, ob->lstyle, ob->lsize, ob->label );
             break;
     }
 
@@ -80,9 +81,9 @@ fl_create_frame( int          type,
     obj = fl_make_object( FL_FRAME, type, x, y, w, h, label, handle_frame );
 
     obj->boxtype = FL_NO_BOX;
-    obj->col1    = FL_FRAME_COL1;
-    obj->col2    = FL_FRAME_COL2;
-    obj->lcol    = FL_FRAME_LCOL;
+    obj->col1    = FLI_FRAME_COL1;
+    obj->col2    = FLI_FRAME_COL2;
+    obj->lcol    = FLI_FRAME_LCOL;
     obj->align   = FL_ALIGN_CENTER;
     obj->active  = 0;
 
