@@ -18,17 +18,10 @@
 /********************** crop here for forms.h **********************/
 
 /**
- * \file Basic.h
- *
  *  Basic definitions and limits.
  *  Window system independent prototypes
- *
- *  Modify with care
  */
 
-#ifndef FL_BASIC_H
-#define FL_BASIC_H
-
 #include <math.h>
 
 #include <X11/Xlib.h>
@@ -36,6 +29,9 @@
 #include <X11/Xatom.h>
 #include <X11/keysym.h>
 #include <X11/Xresource.h>
+#if FL_ENABLE_XFT
+#include <X11/Xft/Xft.h>
+#endif
 
 #if defined __GNUC__
 #define FL_UNUSED_ARG __attribute__ ((unused))
@@ -61,7 +57,7 @@ enum {
     FL_IGNORE      = -1
 };
 
-/* Max  directory length  */
+/* Max. directory length  */
 
 #ifndef FL_PATH_MAX
 #ifndef PATH_MAX
@@ -127,16 +123,14 @@ typedef enum {
     FL_MENUBAR,            /* 33 */
     FL_TEXTBOX,            /* 34, for internal use only */
     FL_LABELBUTTON,        /* 35 */
-    FL_COMBOBOX,           /* 36 */
-    FL_IMAGECANVAS,        /* 37 */
-    FL_THUMBWHEEL,         /* 38 */
-    FL_COLORWHEEL,         /* 39 */
-    FL_FORMBROWSER,        /* 40 */
-    FL_SELECT,             /* 41 */
-    FL_NMENU,              /* 42 */
-    FL_SPINNER,            /* 43 */
-    FL_TBOX,               /* 44 */
-    FL_CLASS_END           /* sentinel */
+    FL_THUMBWHEEL,         /* 36 */
+    FL_COLORWHEEL,         /* 37 */
+    FL_FORMBROWSER,        /* 38 */
+    FL_SELECT,             /* 39 */
+    FL_NMENU,              /* 40 */
+    FL_SPINNER,            /* 41 */
+    FL_TBOX,               /* 42 */
+    FL_CLASS_END
 } FL_CLASS;
 
 #define FL_BEGIN_GROUP        10000
@@ -152,8 +146,8 @@ typedef enum {
 /* How to display a form onto screen */
 
 typedef enum {
-    FL_PLACE_FREE       =   0,      /* size remain resizable      */
-    FL_PLACE_MOUSE      =   1,      /* mouse centered on form     */
+    FL_PLACE_FREE       =   0,         /* size remain resizable      */
+    FL_PLACE_MOUSE      =   1,         /* mouse centered on form     */
     FL_PLACE_CENTER     =   2,         /* center of the screen       */
     FL_PLACE_POSITION   =   4,         /* specific position          */
     FL_PLACE_SIZE       =   8,         /* specific size              */
@@ -179,7 +173,7 @@ enum {
     FL_FULLBORDER = 1,      /* normal                                  */
     FL_TRANSIENT,           /* set TRANSIENT_FOR property              */
     FL_NOBORDER             /* use override_redirect to supress decor. */
- };
+};
 
 /* All box types */
 
@@ -210,7 +204,7 @@ typedef enum {
     FL_BOTTOMTAB_UPBOX,
     FL_SELECTED_BOTTOMTAB_UPBOX,
  
-    FL_MAX_BOX_STYLES               /* sentinel */
+    FL_MAX_BOX_STYLES
 } FL_BOX_TYPE;
 
 #define FL_IS_UPBOX( t )    (    ( t ) == FL_UP_BOX           \
@@ -283,6 +277,14 @@ enum {
 #define FL_SCROLLUPMOUSE     FL_SCROLLUP_MOUSE
 #define FL_SCROLLDOWNMOUSE   FL_SCROLLDOWN_MOUSE
 
+enum {
+    FL_MBUTTON1_BIT =  1,
+    FL_MBUTTON2_BIT =  2,
+    FL_MBUTTON3_BIT =  4,
+    FL_MBUTTON4_BIT =  8,
+    FL_MBUTTON5_BIT = 16
+};
+
 /* Flags for when to return an object */
 
 #define FL_RETURN_NONE            0U
@@ -296,7 +298,7 @@ enum {
 
 
 /*  Some special color indices for FL private colormap. It does not matter
- *  what the value of each enum is, but it must start from 0 and be
+ *  what the value of each enum is, but they must start from 0 and be
  *  consecutive. */
 
 typedef enum {
@@ -497,7 +499,7 @@ typedef enum {
 
 #define FL_DOGERBLUE        FL_DODGERBLUE
 
-/* Events that a form reacts to  */
+/* Events that a form reacts to */
 
 typedef enum {
     FL_NOEVENT,                /*  0 No event */
@@ -541,7 +543,7 @@ typedef enum {
 } FL_EVENTS;
 
 
-/* Resize policies */
+/* Resizing policies */
 
 typedef enum {
     FL_RESIZE_NONE,
@@ -599,10 +601,9 @@ typedef struct {
  * FONTS
  ******************************************************************/
 
-#define FL_MAXFONTS     48  /* max number of fonts */
-
 typedef enum {
     FL_INVALID_STYLE = -1,
+
     FL_NORMAL_STYLE,
     FL_BOLD_STYLE,
     FL_ITALIC_STYLE,
@@ -614,22 +615,34 @@ typedef enum {
     FL_FIXEDBOLDITALIC_STYLE,
 
     FL_TIMES_STYLE,
+	FL_SERIF_STYLE = FL_TIMES_STYLE,
     FL_TIMESBOLD_STYLE,
+	FL_SERIFBOLD_STYLE = FL_TIMESBOLD_STYLE,
     FL_TIMESITALIC_STYLE,
+	FL_SERIFITALIC_STYLE = FL_TIMESITALIC_STYLE,
     FL_TIMESBOLDITALIC_STYLE,
+	FL_SERIFBOLDITALIC_STYLE = FL_TIMESBOLDITALIC_STYLE,
 
     FL_MISC_STYLE,
     FL_MISCBOLD_STYLE,
     FL_MISCITALIC_STYLE,
     FL_SYMBOL_STYLE,
+    FL_MISCBOLDITALIC_STYLE = FL_SYMBOL_STYLE,
 
-    /* modfier masks. Need to fit a short  */
+    /* Modfier masks, need to fit a (signed) int */
 
-    FL_SHADOW_STYLE   = ( 1 <<  9 ),
-    FL_ENGRAVED_STYLE = ( 1 << 10 ),
-    FL_EMBOSSED_STYLE = ( 1 << 11 )
+    FL_SHADOW_STYLE   = ( 1 << 12 ),
+    FL_ENGRAVED_STYLE = ( 1 << 13 ),
+    FL_EMBOSSED_STYLE = ( 1 << 14 )
 } FL_TEXT_STYLE;
 
+
+#if FL_ENABLE_XFT
+#define FL_MAXFONTS     ( FL_SHADOW_STYLE - 1 )
+#else
+#define FL_MAXFONTS     48  /* max number of fonts */
+#endif
+
 #define FL_FONT_STYLE FL_TEXT_STYLE
 
 #define special_style( a )  (    ( a ) >= FL_SHADOW_STYLE                     \
@@ -637,12 +650,14 @@ typedef enum {
 
 /* Standard sizes in XForms */
 
-#define FL_TINY_SIZE        8
-#define FL_SMALL_SIZE      10
-#define FL_NORMAL_SIZE     12
-#define FL_MEDIUM_SIZE     14
-#define FL_LARGE_SIZE      18
-#define FL_HUGE_SIZE       24
+typedef enum {
+    FL_TINY_SIZE   =  8,
+    FL_SMALL_SIZE  = 10,
+    FL_NORMAL_SIZE = 12,
+    FL_MEDIUM_SIZE = 14,
+    FL_LARGE_SIZE  = 18,
+    FL_HUGE_SIZE   = 24
+} FL_DEFAULT_FONT_SIZES;
 
 #define FL_DEFAULT_SIZE   FL_SMALL_SIZE
 
@@ -659,7 +674,7 @@ typedef enum {
 #define FL_NORMAL_FONT2   FL_NORMAL_FONT
 #define FL_DEFAULT_FONT   FL_SMALL_FONT
 
-#define FL_BOUND_WIDTH  ( FL_Coord ) 1     /* Border width of boxes */
+#define FL_BOUND_WIDTH  ( ( FL_Coord ) 1 )    /* Border width of boxes */
 
 /* Definition of basic struct that holds an object */
 
@@ -725,6 +740,9 @@ struct FL_OBJECT_ {
     void             ( * set_return )( FL_OBJECT *,
                                        unsigned int );
 
+    unsigned int     ( * set_react_to )( FL_OBJECT *,
+                                         unsigned int );
+
     /* Re-configure preference */
 
     unsigned int     resize;         /* what to do if WM resizes the FORM     */
@@ -739,7 +757,7 @@ struct FL_OBJECT_ {
     FL_OBJECT      * nc;             /* next child */
 
     FL_pixmap      * flpixmap;       /* pixmap double buffering stateinfo */
-    int              use_pixmap;     /* true to use pixmap double buffering*/
+    int              use_pixmap;     /* if true use pixmap double buffering*/
 
     /* Some interaction flags */
 
@@ -757,11 +775,12 @@ struct FL_OBJECT_ {
     int              redraw;
     int              visible;
     int              is_under;       /* if (partially) hidden by other object */
-    int              clip;
+    unsigned int     react_to;       /* mouse buttons to react to */
     unsigned long    click_timeout;
     void           * c_vdata;        /* for class use */
     char           * c_cdata;        /* for class use */
     long             c_ldata;        /* for class use */
+	void           * fdesign_vdata;  /* for fdesigns exclusive use */
     FL_COLOR         dbl_background; /* double buffer background */
     char           * tooltip;
     int              tipID;
@@ -770,7 +789,6 @@ struct FL_OBJECT_ {
     int              want_update;
 };
 
-
 /* Callback function for an entire form */
 
 typedef void ( * FL_FORMCALLBACKPTR )( FL_OBJECT *,
@@ -1341,6 +1359,12 @@ FL_EXPORT void fl_deactivate_object( FL_OBJECT * ob );
 
 FL_EXPORT int fl_object_is_active( FL_OBJECT * obj );
 
+FL_EXPORT unsigned int
+    fl_set_object_mouse_buttons( FL_OBJECT    * obj,
+								 unsigned int   mouse_buttons );
+
+FL_EXPORT unsigned int fl_get_object_mouse_buttons( FL_OBJECT * obj );
+
 FL_EXPORT int fl_enumerate_fonts( void ( * output )( const char * s ),
                                   int  shortform );
 
@@ -1438,7 +1462,7 @@ FL_EXPORT void fl_draw_text_cursor( int          align,
 									int          style,
 									int          size,
 									const char * str,
-									int          cc,
+									FL_COLOR     cc,
 									int          pos );
 
 #define fl_draw_box fl_draw_box
@@ -1628,5 +1652,3 @@ typedef const char * ( * FL_VAL_FILTER )( FL_OBJECT *,
 FL_EXPORT int fl_is_same_object( FL_OBJECT * obj1,
 								 FL_OBJECT * obj2 );
 
-#endif /* ! defined FL_BASIC_H */
-
