@@ -24,9 +24,6 @@
  *
  */
 
-#ifndef FL_XBASIC_H
-#define FL_XBASIC_H
-
 /* Draw mode */
 
 enum {
@@ -87,7 +84,11 @@ enum {
 
 typedef struct {
     XVisualInfo   * xvinfo;
+#if FL_ENABLE_XFT
+    XftFont       * cur_fnt;            /* current font */
+#else
     XFontStruct   * cur_fnt;            /* current font in default GC */
+#endif
     Colormap        colormap;           /* colormap valid for xvinfo */
     Window          trailblazer;        /* a valid window for xvinfo */
     int             vclass,             /* visual class and color depth */
@@ -95,8 +96,13 @@ typedef struct {
     int             rgb_bits;           /* primary color resolution */
     int             dithered;           /* true if dithered color */
     int             pcm;                /* true if colormap is not shared */
-    GC              gc[ 16 ];           /* working GC */
-    GC              textgc[ 16 ];       /* GC used exclusively for text */
+    GC              gc;                 /* working GC */
+#if FL_ENABLE_XFT
+	XftDraw       * textdraw;           /* for drawing texts */
+	XftDraw       * bgdraw;             /* for drawing background of text */
+#else
+    GC              textgc;             /* GC used exclusively for text */
+#endif
     GC              dimmedGC;           /* A GC having a checkboard stipple */
     unsigned long   lut[ FL_MAX_COLS ]; /* secondary lookup table */
     unsigned int    rshift,
@@ -142,6 +148,9 @@ FL_EXPORT FL_State fl_state[ ];
 
 FL_EXPORT char *fl_ul_magic_char;
 
+FL_EXPORT double fl_dpi;
+#define fl_get_dpi( ) ( fl_dpi )
+
 FL_EXPORT int fl_mode_capable( int mode,
                                int warn );
 
@@ -170,6 +179,45 @@ struct FL_pixmap_ {
 
 /* Fonts related */
 
+#if FL_ENABLE_XFT
+
+typedef struct {
+	int       size;
+	XftFont * font;
+} FL_SIZED_FONT;
+
+typedef struct {
+    size_t          index;
+	char          * name;
+	int             slant;
+	int             weight;
+	double          scale;
+	FL_SIZED_FONT * sized_fonts;
+	size_t          size_count;
+} FL_FONT;
+
+typedef enum {
+	FL_SLANT_ROMAN   = XFT_SLANT_ROMAN,
+	FL_SLANT_ITALIC  = XFT_SLANT_ITALIC,
+	FL_SLANT_OBLIQUE = XFT_SLANT_OBLIQUE
+} FL_FONT_SLANT;
+
+typedef enum {
+	FL_WEIGHT_LIGHT    = XFT_WEIGHT_LIGHT,
+	FL_WEIGHT_MEDIUM   = XFT_WEIGHT_MEDIUM,
+	FL_WEIGHT_DEMIBOLD = XFT_WEIGHT_DEMIBOLD,
+	FL_WEIGHT_BOLD     = XFT_WEIGHT_BOLD,
+	FL_WEIGHT_BLACK    = XFT_WEIGHT_BLACK
+} FL_FONT_WEIGHT;
+
+typedef enum {
+	FL_X11_FONT,
+	FL_XFT_FONT
+} FL_FONT_TYPE;
+
+
+#else  /* X11 fonts */
+
 #define FL_MAX_FONTSIZES         10
 #define FL_MAX_FONTNAME_LENGTH   80
 
@@ -180,6 +228,8 @@ typedef struct {
     char          fname[ FL_MAX_FONTNAME_LENGTH + 1 ];  /* without size info */
 } FL_FONT;
 
+#endif
+
 /* Some basic drawing routines */
 
 typedef XPoint      FL_POINT;
@@ -374,10 +424,31 @@ FL_EXPORT void fl_draw_frame( int      style,
  * Interfaces
  */
 
-FL_EXPORT XFontStruct *fl_get_fontstruct( int style,
-                                          int size );
+#if FL_ENABLE_XFT
+FL_EXPORT XftFont * fl_get_font_struct( int style,
+										int size );
 
-#define fl_get_font_struct    fl_get_fontstruct
+FL_EXPORT int fl_set_xft_font_name( int              index,
+									const char     * name,
+									FL_FONT_SLANT    slant,
+									FL_FONT_WEIGHT   weight );
+
+FL_EXPORT void fl_set_default_font_type( FL_FONT_TYPE type );
+
+FL_EXPORT int fl_unload_font( int index );
+
+FL_EXPORT void fl_set_global_font_scaling_factor( double factor );
+
+FL_EXPORT void fl_set_font_scaling_factor( int    index,
+										   double factor );
+
+#else
+FL_EXPORT XFontStruct *fl_get_font_struct( int style,
+										   int size );
+
+#endif
+
+#define fl_get_fontstruct     fl_get_fonts_truct
 #define fl_get_fntstruct      fl_get_font_struct
 
 FL_EXPORT Window fl_get_mouse( FL_Coord  * x,
@@ -881,13 +952,3 @@ typedef struct {
 } FL_RGB2PIXEL_;
 
 #define FL_RGB2PIXEL  FL_RGB2PIXEL_
-
-#endif /* ! defined FL_XBASIC_H */
-
-
-/*
- * Local variables:
- * tab-width: 4
- * indent-tabs-mode: nil
- * End:
- */
