@@ -64,8 +64,8 @@ static int make_line_visible( FL_OBJECT * obj,
 static int make_char_visible( FL_OBJECT * obj,
                               int         xpos );
 
-static void copy_attributes( FL_OBJECT *,
-                             FL_OBJECT * );
+static void copy_attributes( FL_OBJECT       * dest,
+                             const FL_OBJECT * src );
 
 static int date_validator( FL_OBJECT *,
                            const char *,
@@ -303,7 +303,7 @@ draw_input( FL_OBJECT * obj )
                                     && sp->beginrange >= sp->endrange ) ?
                                   sp->position : -1,
                                   sp->beginrange, sp->endrange,
-                                 sp->str, sp->drawtype != COMPLETE,
+                                  sp->str, sp->drawtype != COMPLETE,
                                   sp->topline,
                                   sp->topline + sp->screenlines, 0 );
 
@@ -1661,11 +1661,11 @@ fl_create_input( int          type,
     set_default_keymap( 0 );
 
     obj = fl_make_object( FL_INPUT, type, x, y, w, h, label, handle_input );
-    obj->boxtype    = FL_INPUT_BOXTYPE;
-    obj->col1       = FL_INPUT_COL1;
-    obj->col2       = FL_INPUT_COL2;
-    obj->align      = FL_INPUT_ALIGN;
-    obj->lcol       = FL_INPUT_LCOL;
+    obj->boxtype    = FLI_INPUT_BOXTYPE;
+    obj->col1       = FLI_INPUT_COL1;
+    obj->col2       = FLI_INPUT_COL2;
+    obj->align      = FLI_INPUT_ALIGN;
+    obj->lcol       = FLI_INPUT_LCOL;
     obj->lsize      = fli_cntl.inputFontSize ?
                       fli_cntl.inputFontSize : FL_DEFAULT_SIZE;
     obj->set_return = fl_set_input_return;
@@ -1680,8 +1680,8 @@ fl_create_input( int          type,
     obj->click_timeout = FL_CLICK_TIMEOUT;
     obj->spec = sp     = fl_calloc( 1, sizeof *sp );
 
-    sp->textcol        = FL_INPUT_TCOL;
-    sp->curscol        = FL_INPUT_CCOL;
+    sp->textcol        = FLI_INPUT_TCOL;
+    sp->curscol        = FLI_INPUT_CCOL;
     sp->position       = -1;
     sp->endrange       = -1;
     sp->size           = 8;
@@ -2919,8 +2919,8 @@ fl_get_input_editkeymap( FL_EditKeymap * keymap )
  ***************************************/
 
 static void
-copy_attributes( FL_OBJECT * dest,
-                 FL_OBJECT * src )
+copy_attributes( FL_OBJECT       * dest,
+                 const FL_OBJECT * src )
 {
     if ( src == dest )
         return;
@@ -3003,7 +3003,7 @@ fl_set_input_cursor_visible( FL_OBJECT * obj,
  ***************************************/
 
 int
-fl_input_changed( FL_OBJECT *obj )
+fl_input_changed( FL_OBJECT * obj )
 {
     return ( ( FLI_INPUT_SPEC * ) obj->spec )->changed;
 }
