@@ -32,6 +32,7 @@
 
 #include "include/forms.h"
 #include "flinternal.h"
+#include "private/pframe.h"
 
 
 /***************************************
@@ -70,8 +71,8 @@ handle_lframe( FL_OBJECT * ob,
             break;
 
         case FL_DRAW :
-            fl_drw_frame( ob->type, ob->x, ob->y, ob->w, ob->h,
-                          ob->col1, ob->bw );
+            fl_draw_frame( ob->type, ob->x, ob->y, ob->w, ob->h,
+                           ob->col1, ob->bw );
             /* fall through */
 
         case FL_DRAWLABEL :
@@ -139,10 +140,9 @@ handle_lframe( FL_OBJECT * ob,
                     break;
             }
 
-            fl_drw_box( FL_FLAT_BOX, sx, sy, sw, sh, ob->col2, 0 );
-
-            fl_drw_text( FL_ALIGN_CENTER, sx, sy, sw, sh,
-                         ob->lcol, ob->lstyle, ob->lsize, ob->label );
+            fl_draw_box( FL_FLAT_BOX, sx, sy, sw, sh, ob->col2, 0 );
+            fl_draw_text( FL_ALIGN_CENTER, sx, sy, sw, sh,
+                          ob->lcol, ob->lstyle, ob->lsize, ob->label );
             break;
     }
 
@@ -166,9 +166,9 @@ fl_create_labelframe( int          type,
     ob = fl_make_object( FL_LABELFRAME, type, x, y, w, h, label,
                          handle_lframe );
     ob->boxtype = FL_NO_BOX;
-    ob->col1    = FL_FRAME_COL1;
-    ob->col2    = FL_FRAME_COL2;
-    ob->lcol    = FL_FRAME_LCOL;
+    ob->col1    = FLI_FRAME_COL1;
+    ob->col2    = FLI_FRAME_COL2;
+    ob->lcol    = FLI_FRAME_LCOL;
     ob->align   = FL_ALIGN_LEFT_TOP;
     ob->active  = 0;
 
