@@ -261,16 +261,16 @@ handle_menu( FL_OBJECT * ob,
             else
                 col = ob->col1;
 
-            fl_drw_box( boxtype, ob->x, ob->y, ob->w, ob->h, col, ob->bw );
-            fl_drw_text( ob->align, ob->x, ob->y, ob->w, ob->h,
-                         ob->lcol, ob->lstyle, ob->lsize, ob->label );
+            fl_draw_box( boxtype, ob->x, ob->y, ob->w, ob->h, col, ob->bw );
+            fl_draw_text( ob->align, ob->x, ob->y, ob->w, ob->h,
+                          ob->lcol, ob->lstyle, ob->lsize, ob->label );
 
             if ( sp->showsymbol )
             {
                 int dm = 0.85 * FL_min( ob->w, ob->h );
 
-                fl_drw_text( 0, ob->x + ob->w - dm - 1, ob->y + 1,
-                             dm, dm, col, 0, 0, "@menu" );
+                fl_draw_text( 0, ob->x + ob->w - dm - 1, ob->y + 1,
+                              dm, dm, col, 0, 0, "@menu" );
             }
             break;
 
@@ -280,11 +280,13 @@ handle_menu( FL_OBJECT * ob,
             break;
 
         case FL_PUSH:
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             /* Touch menus and push menus without a title don't do anything
                on a button press */
 
-            if (    key != FL_MBUTTON1
-                 || ( ob->type == FL_PUSH_MENU && sp->no_title ) )
+            if ( ob->type == FL_PUSH_MENU && sp->no_title )
                 break;
 
             if ( ob->type == FL_TOUCH_MENU )
@@ -299,13 +301,15 @@ handle_menu( FL_OBJECT * ob,
             break;
 
         case FL_RELEASE :
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             /* Button release is only important for push menus without a
                title, all others get started by a button press or by just
                moving the mouse on top of it (and they also don't expect
                a release - that gets eaten by the popup handler) */
 
-            if (    key != FL_MBUTTON1
-                 || ! ( ob->type == FL_PUSH_MENU && sp->no_title )
+            if (    ! ( ob->type == FL_PUSH_MENU && sp->no_title )
                  || mx < ob->x
                  || mx > ob->x + ob->w
                  || my < ob->y
@@ -360,11 +364,11 @@ fl_create_menu( int          type,
     obj = fl_make_object( FL_MENU, type, x, y, w, h, label, handle_menu );
 
     obj->boxtype = FL_FLAT_BOX;
-    obj->col1   = FL_MENU_COL1;
-    obj->col2   = FL_MENU_COL2;
-    obj->lcol   = FL_MENU_LCOL;
+    obj->col1   = FLI_MENU_COL1;
+    obj->col2   = FLI_MENU_COL2;
+    obj->lcol   = FLI_MENU_LCOL;
     obj->lstyle = FL_NORMAL_STYLE;
-    obj->align  = FL_MENU_ALIGN;
+    obj->align  = FLI_MENU_ALIGN;
 
     if ( type == FL_TOUCH_MENU )
         fl_set_object_return( obj, FL_RETURN_CHANGED );
@@ -374,6 +378,8 @@ fl_create_menu( int          type,
     sp = obj->spec = fl_calloc( 1, sizeof *sp );
     sp->extern_menu = -1;
 
+    fl_set_object_mouse_buttons( obj, FL_MBUTTON1_BIT );
+
     return obj;
 }
 
@@ -445,7 +451,7 @@ addto_menu( FL_OBJECT  * ob,
     char *p;
 
     if (    ISPUP( sp )
-         || sp->numitems >= FL_MENU_MAXITEMS
+         || sp->numitems >= FLI_MENU_MAXITEMS
          || sp->cur_val == INT_MAX )
         return;
 
@@ -526,7 +532,7 @@ fl_set_menu( FL_OBJECT  * ob,
     t = fl_strdup( menustr );
 
     for ( c = strtok( t, "|" );
-          c && sp->numitems < FL_CHOICE_MAXITEMS;
+          c && sp->numitems < FLI_MENU_MAXITEMS;
           c = strtok( NULL, "|" ) )
     {
         FL_PUP_CB cb;
@@ -576,7 +582,7 @@ fl_addto_menu( FL_OBJECT  * ob,
     t = fl_strdup( menustr );
 
     for ( c = strtok( t, "|" );
-          c && sp->numitems < FL_CHOICE_MAXITEMS;
+          c && sp->numitems < FLI_MENU_MAXITEMS;
           c = strtok( NULL, "|" ) )
     {
         FL_PUP_CB cb;
