@@ -33,7 +33,7 @@
 
 #include "include/forms.h"
 #include "flinternal.h"
-#include XPM_H_LOCATION
+#include "private/ppixmap.h"
 
 
 #if FL_DEBUG >= ML_ERR
@@ -50,17 +50,6 @@
 #endif
 
 
-typedef struct {
-    XpmAttributes * xpma;
-    GC              gc;
-    int             align;
-    int             dx,
-                    dy;
-    int             show_focus;
-    unsigned int    focus_w,    /* these should be in button_spec */
-                    focus_h;
-} PixmapSPEC;
-
 static XpmAttributes *xpmattrib;
 
 
@@ -103,9 +92,10 @@ cleanup_xpma_struct( XpmAttributes * xpma )
  ***************************************/
 
 static void
-free_pixmap( FL_BUTTON_STRUCT * sp )
+free_pixmap( FL_OBJECT * obj )
 {
-    PixmapSPEC *psp = sp->cspecv;
+    FLI_BUTTON_SPEC *sp = obj->spec;
+    FLI_PIXMAP_SPEC *psp = obj->c_vdata;
 
     fl_free_pixmap( sp->pixmap );
     fl_free_pixmap( sp->mask );
@@ -121,7 +111,7 @@ free_pixmap( FL_BUTTON_STRUCT * sp )
  ***************************************/
 
 static void
-free_focuspixmap( FL_BUTTON_STRUCT * sp )
+free_focuspixmap( FLI_BUTTON_SPEC * sp )
 {
     fl_free_pixmap( sp->focus_pixmap );
     fl_free_pixmap( sp->focus_mask );
@@ -136,19 +126,20 @@ free_focuspixmap( FL_BUTTON_STRUCT * sp )
  ***************************************/
 
 static void
-change_pixmap( FL_BUTTON_STRUCT * sp,
-               Window             win,
-               Pixmap             p,
-               Pixmap             shape_mask,
-               int                del )
+change_pixmap( FL_OBJECT * obj,
+               Window      win,
+               Pixmap      p,
+               Pixmap      shape_mask,
+               int         del )
 {
-    PixmapSPEC *psp = sp->cspecv;
+    FLI_BUTTON_SPEC *sp = obj->spec;
+    FLI_PIXMAP_SPEC *psp = obj->c_vdata;
 
     if ( p == None || win == None )
         return;
 
     if ( del )
-        free_pixmap( sp );
+        free_pixmap( obj );
     else
     {
         cleanup_xpma_struct( psp->xpma );
@@ -176,11 +167,11 @@ change_pixmap( FL_BUTTON_STRUCT * sp,
  ***************************************/
 
 static void
-change_focuspixmap( FL_BUTTON_STRUCT * sp,
-                    Window             win  FL_UNUSED_ARG,
-                    Pixmap             p,
-                    Pixmap             shape_mask,
-                    int                del )
+change_focuspixmap( FLI_BUTTON_SPEC * sp,
+                    Window            win  FL_UNUSED_ARG,
+                    Pixmap            p,
+                    Pixmap            shape_mask,
+                    int               del )
 {
     if ( del )
         free_focuspixmap( sp );
@@ -197,8 +188,8 @@ static void
 show_pixmap( FL_OBJECT * obj,
              int         focus )
 {
-    FL_BUTTON_STRUCT *sp = obj->spec;
-    PixmapSPEC *psp = sp->cspecv;
+    FLI_BUTTON_SPEC *sp = obj->spec;
+    FLI_PIXMAP_SPEC *psp = obj->c_vdata;
     int dest_x,
         dest_y,
         dest_w,
@@ -227,8 +218,8 @@ show_pixmap( FL_OBJECT * obj,
 
     if ( pixmap == None || bits_w == 0 || bits_h == 0 )
     {
-        fl_drw_text( FL_ALIGN_CENTER, obj->x, obj->y, obj->w, obj->h,
-                     obj->lcol, obj->lstyle, FL_TINY_SIZE, "p" );
+        fl_draw_text( FL_ALIGN_CENTER, obj->x, obj->y, obj->w, obj->h,
+                      obj->lcol, obj->lstyle, FL_TINY_SIZE, "p" );
         return;
     }
 
@@ -390,8 +381,8 @@ draw_pixmap( FL_OBJECT * obj )
 {
     /* Draw the box */
 
-    fl_drw_box( obj->boxtype, obj->x, obj->y, obj->w, obj->h,
-                obj->col2, obj->bw );
+    fl_draw_box( obj->boxtype, obj->x, obj->y, obj->w, obj->h,
+                 obj->col2, obj->bw );
     show_pixmap( obj, 0 );
 }
 
@@ -407,8 +398,6 @@ handle_pixmap( FL_OBJECT * obj,
                int         key  FL_UNUSED_ARG,
                void      * ev   FL_UNUSED_ARG )
 {
-    FL_BUTTON_STRUCT *sp = obj->spec;
-
 #if FL_DEBUG >= ML_DEBUG
     M_info( "handle_pixmap", fli_event_name( event ) );
 #endif
@@ -424,10 +413,11 @@ handle_pixmap( FL_OBJECT * obj,
             break;
 
         case FL_FREEMEM:
-            free_pixmap( obj->spec );
-            if ( ( ( PixmapSPEC * ) sp->cspecv )->gc )
-                XFreeGC( flx->display, ( ( PixmapSPEC * ) sp->cspecv )->gc );
-            fli_safe_free( sp->cspecv );
+            free_pixmap( obj );
+            if ( ( ( FLI_PIXMAP_SPEC * ) obj->c_vdata )->gc )
+                XFreeGC( flx->display,
+                         ( ( FLI_PIXMAP_SPEC * ) obj->c_vdata )->gc );
+            fli_safe_free( obj->c_vdata );
             fli_safe_free( obj->spec );
             break;
     }
@@ -449,21 +439,22 @@ fl_create_pixmap( int          type,
                   const char * label )
 {
     FL_OBJECT *obj;
-    FL_BUTTON_STRUCT *sp;
-    PixmapSPEC *psp;
+    FLI_BUTTON_SPEC *sp;
+    FLI_PIXMAP_SPEC *psp;
 
     obj = fl_make_object( FL_PIXMAP, type, x, y, w, h, label, handle_pixmap );
 
-    obj->boxtype = FL_BITMAP_BOXTYPE;
-    obj->col1    = FL_BITMAP_COL1;
-    obj->col2    = FL_BITMAP_COL2;
-    obj->lcol    = FL_BITMAP_LCOL;
-    obj->align   = FL_BITMAP_ALIGN;
-    obj->active  = type != FL_NORMAL_BITMAP;
+    obj->boxtype = FLI_PIXMAP_BOXTYPE;
+    obj->col1    = FLI_PIXMAP_COL1;
+    obj->col2    = FLI_PIXMAP_COL2;
+    obj->lcol    = FLI_PIXMAP_LCOL;
+    obj->align   = FLI_PIXMAP_ALIGN;
+    obj->active  = type != FL_NORMAL_PIXMAP;
     obj->spec    = sp = fl_calloc( 1, sizeof *sp );
 
     sp->bits_w = 0;
-    sp->cspecv = psp = fl_calloc( 1, sizeof *psp );
+
+    obj->c_vdata = psp = fl_calloc( 1, sizeof *psp );
 
     psp->dx    = psp->dy = 0;
     psp->align = FL_ALIGN_CENTER;
@@ -554,14 +545,14 @@ fl_set_pixmap_pixmap( FL_OBJECT * obj,
                       Pixmap      id,
                       Pixmap      mask )
 {
-    FL_BUTTON_STRUCT *sp;
+    FLI_BUTTON_SPEC *sp;
     FL_Coord w = 0,
              h = 0;
 
     CHECK( obj, "fl_set_pixmap_pixmap" );
 
     sp = obj->spec;
-    change_pixmap( sp, FL_ObjWin( obj ), id, mask, 0 ); /* 0 don't free old */
+    change_pixmap( obj, FL_ObjWin( obj ), id, mask, 0 ); /* 0 don't free old */
 
     if ( sp->pixmap != None )
         fl_get_winsize( sp->pixmap, &w, &h );
@@ -581,7 +572,7 @@ fl_get_pixmap_pixmap( FL_OBJECT * obj,
                       Pixmap    * p,
                       Pixmap    * m )
 {
-    FL_BUTTON_STRUCT *sp;
+    FLI_BUTTON_SPEC *sp;
 
     if (    ! IsValidClass( obj, FL_PIXMAP )
          && ! IsValidClass( obj, FL_PIXMAPBUTTON ) )
@@ -667,7 +658,7 @@ fl_set_pixmap_file( FL_OBJECT  * obj,
 {
     Pixmap p = None,
            shape_mask = None;
-    FL_BUTTON_STRUCT *sp;
+    FLI_BUTTON_SPEC *sp;
     int hotx, hoty;
     Window win;
 
@@ -683,8 +674,8 @@ fl_set_pixmap_file( FL_OBJECT  * obj,
 
     if ( p != None )
     {
-        change_pixmap( sp, win, p, shape_mask, 0 );
-        ( ( PixmapSPEC * ) sp->cspecv )->xpma = xpmattrib;
+        change_pixmap( obj, win, p, shape_mask, 0 );
+        ( ( FLI_PIXMAP_SPEC * ) obj->c_vdata )->xpma = xpmattrib;
         fl_redraw_object( obj );
     }
 }
@@ -706,18 +697,18 @@ fl_set_pixmap_file( FL_OBJECT  * obj,
  ***************************************/
 
 static void
-draw_pixmapbutton( FL_OBJECT * obj )
+draw_pixmapbutton( FL_OBJECT * obj,
+                   int         event )
 {
-    FL_BUTTON_STRUCT *sp = obj->spec;
-    PixmapSPEC *psp = sp->cspecv;
+    FLI_PIXMAP_SPEC *psp = obj->c_vdata;
 
     /* Draw it like a "normal button */
 
-    fli_draw_button( obj );
+    draw_normal_button( obj, event );
 
     /* Add the pixmap on top of it */
 
-    switch ( sp->event )
+    switch ( event )
     {
         case FL_ENTER:
             if ( psp->show_focus )
@@ -744,9 +735,9 @@ draw_pixmapbutton( FL_OBJECT * obj )
  ***************************************/
 
 static void
-cleanup_pixmapbutton( FL_BUTTON_STRUCT *sp )
+cleanup_pixmapbutton( FL_OBJECT * obj )
 {
-    PixmapSPEC *psp = sp->cspecv;
+    FLI_PIXMAP_SPEC *psp = obj->c_vdata;
 
     if ( psp->gc )
     {
@@ -760,11 +751,8 @@ cleanup_pixmapbutton( FL_BUTTON_STRUCT *sp )
         psp->xpma = NULL;
     }
 
-    if ( sp->cspecv )
-    {
-        fl_free( sp->cspecv );
-        sp->cspecv = NULL;
-    }
+    if ( obj->c_vdata )
+        fli_safe_free( obj->c_vdata );
 }
 
 
@@ -781,8 +769,7 @@ fl_create_pixmapbutton( int          type,
 {
     FL_OBJECT *obj;
     static int class_init;
-    FL_BUTTON_STRUCT *sp;
-    PixmapSPEC *psp;
+    FLI_PIXMAP_SPEC *psp;
 
     if ( ! class_init )
     {
@@ -793,15 +780,13 @@ fl_create_pixmapbutton( int          type,
 
     obj = fl_create_generic_button( FL_PIXMAPBUTTON, type, x, y, w, h, label );
 
-    obj->boxtype = FL_PIXMAPBUTTON_BOXTYPE;
-    obj->col1    = FL_PIXMAPBUTTON_COL1;
-    obj->col2    = FL_PIXMAPBUTTON_COL2;
-    obj->align   = FL_PIXMAPBUTTON_ALIGN;
-    obj->lcol    = FL_PIXMAPBUTTON_LCOL;
+    obj->boxtype = FLI_PIXMAPBUTTON_BOXTYPE;
+    obj->col1    = FLI_PIXMAPBUTTON_COL1;
+    obj->col2    = FLI_PIXMAPBUTTON_COL2;
+    obj->align   = FLI_PIXMAPBUTTON_ALIGN;
+    obj->lcol    = FLI_PIXMAPBUTTON_LCOL;
 
-    sp = obj->spec;   /* allocated in fl_create_generic_button() */
-
-    sp->cspecv = psp = fl_calloc( 1, sizeof *psp );
+    obj->c_vdata = psp = fl_calloc( 1, sizeof *psp );
 
     psp->show_focus = 1;
     psp->align      = FL_ALIGN_CENTER;
@@ -839,7 +824,7 @@ void
 fl_set_pixmap_data( FL_OBJECT   * obj,
                     char       ** bits )
 {
-    FL_BUTTON_STRUCT *sp;
+    FLI_BUTTON_SPEC *sp;
     Window win;
     Pixmap p,
            shape_mask = None;
@@ -857,8 +842,8 @@ fl_set_pixmap_data( FL_OBJECT   * obj,
 
     if ( p != None )
     {
-        change_pixmap( sp, win, p, shape_mask, 0 );
-        ( ( PixmapSPEC * ) sp->cspecv )->xpma = xpmattrib;
+        change_pixmap( obj, win, p, shape_mask, 0 );
+        ( ( FLI_PIXMAP_SPEC * ) obj->c_vdata )->xpma = xpmattrib;
         fl_redraw_object( obj );
     }
 
@@ -891,13 +876,11 @@ fl_set_pixmap_align( FL_OBJECT * obj,
                      int         xmargin,
                      int         ymargin )
 {
-    FL_BUTTON_STRUCT *sp;
-    PixmapSPEC *psp;
+    FLI_PIXMAP_SPEC *psp;
 
     CHECK( obj, "fl_set_pixmap_align" );
 
-    sp = obj->spec;
-    psp = sp->cspecv;
+    psp = obj->c_vdata;
     if ( align != psp->align || xmargin != psp->dx || ymargin != psp->dy )
     {
         psp->align = align;
@@ -916,8 +899,8 @@ fl_set_pixmapbutton_focus_pixmap( FL_OBJECT * obj,
                                   Pixmap      id,
                                   Pixmap      mask )
 {
-    FL_BUTTON_STRUCT *sp = obj->spec;
-    PixmapSPEC *psp = sp->cspecv;
+    FLI_BUTTON_SPEC *sp = obj->spec;
+    FLI_PIXMAP_SPEC *psp = obj->c_vdata;
     int w,
         h;
 
@@ -940,8 +923,8 @@ void
 fl_set_pixmapbutton_focus_data( FL_OBJECT  * obj,
                                 char      ** bits )
 {
-    FL_BUTTON_STRUCT *sp;
-    PixmapSPEC *psp;
+    FLI_BUTTON_SPEC *sp;
+    FLI_PIXMAP_SPEC *psp;
     Window win;
     Pixmap p,
            shape_mask = None;
@@ -954,7 +937,7 @@ fl_set_pixmapbutton_focus_data( FL_OBJECT  * obj,
         return;
 
     sp = obj->spec;
-    psp = sp->cspecv;
+    psp = obj->c_vdata;
     win = FL_ObjWin( obj ) ? FL_ObjWin( obj ) : fl_default_win( );
     p = fl_create_from_pixmapdata( win, bits, &psp->focus_w, &psp->focus_h,
                                    &shape_mask, &hx, &hy, obj->col1 );
@@ -962,7 +945,7 @@ fl_set_pixmapbutton_focus_data( FL_OBJECT  * obj,
     if ( p != None )
     {
         change_focuspixmap( sp, win, p, shape_mask, 0 );
-        ( ( PixmapSPEC * ) sp->cspecv )->xpma = xpmattrib;
+        ( ( FLI_PIXMAP_SPEC * ) obj->c_vdata )->xpma = xpmattrib;
     }
 }
 
@@ -976,17 +959,17 @@ fl_set_pixmapbutton_focus_file( FL_OBJECT  * obj,
 {
     Pixmap p,
            shape_mask = None;
-    FL_BUTTON_STRUCT *sp;
+    FLI_BUTTON_SPEC *sp;
     int hotx,
         hoty;
     Window win;
-    PixmapSPEC *psp;
+    FLI_PIXMAP_SPEC *psp;
 
     if ( ! flx->display )
         return;
 
     sp = obj->spec;
-    psp = sp->cspecv;
+    psp = obj->c_vdata;
     win = FL_ObjWin( obj ) ? FL_ObjWin( obj ) : fl_default_win( );
     p = fl_read_pixmapfile( win, fname, &psp->focus_w, &psp->focus_h,
                             &shape_mask, &hotx, &hoty, obj->col1 );
@@ -1006,13 +989,11 @@ void
 fl_set_pixmapbutton_focus_outline( FL_OBJECT * obj,
                                    int         yes_no )
 {
-    FL_BUTTON_STRUCT *sp;
-    PixmapSPEC *psp;
+    FLI_PIXMAP_SPEC *psp;
 
     CHECK( obj, "fl_set_pixmapbutton_focus_outline" );
 
-    sp = obj->spec;
-    psp = sp->cspecv;
+    psp = obj->c_vdata;
     psp->show_focus = yes_no;
 }
 
@@ -1025,7 +1006,7 @@ fl_free_pixmap_pixmap( FL_OBJECT * obj )
 {
     CHECK( obj, "fl_free_pixmap_pixmap" );
 
-    free_pixmap( obj->spec );
+    free_pixmap( obj );
 }
 
 
@@ -1047,8 +1028,8 @@ fl_free_pixmap_focus_pixmap( FL_OBJECT * obj )
  ***************************************/
 
 void
-fli_set_form_icon_data( FL_FORM  * form,
-                        char    ** data )
+fl_set_form_icon_data( FL_FORM  * form,
+                       char    ** data )
 {
     Pixmap p,
            s = None;
@@ -1061,6 +1042,8 @@ fli_set_form_icon_data( FL_FORM  * form,
         fl_set_form_icon( form, p, s );
         fl_free( xpmattrib );
     }
+    else
+        M_warn( "fl_set_form_icon_data", "Failed to create pixnap" );
 }
 
 
