@@ -131,29 +131,29 @@ draw_motion( FL_OBJECT * ob )
         }
 
         XSetClipRectangles( flx->display, flx->gc, 0, 0, xrec, 2, Unsorted );
-        fl_drw_box( FL_FLAT_BOX, ob->x + sp->x + abbw, ob->y + sp->y + abbw,
-                    sp->w - 2 * abbw, sp->h - 2 * abbw, ob->col1, 0 );
+        fl_draw_box( FL_FLAT_BOX, ob->x + sp->x + abbw, ob->y + sp->y + abbw,
+                     sp->w - 2 * abbw, sp->h - 2 * abbw, ob->col1, 0 );
     }
     else if (    ob->type == FL_HOR_THIN_SLIDER
               || ob->type == FL_VERT_THIN_SLIDER
               || ob->type == FL_HOR_BASIC_SLIDER
               || ob->type == FL_VERT_BASIC_SLIDER )
-        fl_drw_box( FL_FLAT_BOX, ob->x + sp->x, ob->y + sp->y,
-                    sp->w, sp->h, ob->col1, 1 );
+        fl_draw_box( FL_FLAT_BOX, ob->x + sp->x, ob->y + sp->y,
+                     sp->w, sp->h, ob->col1, 1 );
     else if (    ob->type == FL_HOR_BROWSER_SLIDER2
               || ob->type == FL_VERT_BROWSER_SLIDER2 )
-        fl_drw_box( ob->boxtype, ob->x + sp->x, ob->y + sp->y,
-                    sp->w, sp->h, ob->col1, ob->bw > 0 ? 1 : -1 );
+        fl_draw_box( ob->boxtype, ob->x + sp->x, ob->y + sp->y,
+                     sp->w, sp->h, ob->col1, ob->bw > 0 ? 1 : -1 );
     else
-        fl_drw_box( FL_UP_BOX, ob->x + sp->x, ob->y + sp->y,
-                    sp->w, sp->h, ob->col1, ob->bw > 0 ? 1 : -1 );
+        fl_draw_box( FL_UP_BOX, ob->x + sp->x, ob->y + sp->y,
+                     sp->w, sp->h, ob->col1, ob->bw > 0 ? 1 : -1 );
 
     fl_unset_clipping( );
 
     col = ( IS_SCROLLBAR( ob ) && sp->mouse == FLI_SLIDER_KNOB ) ?
           FL_MCOL : ob->col2;
 
-    fli_drw_slider( ob, ob->col1, col, NULL, FLI_SLIDER_KNOB );
+    fli_draw_slider( ob, ob->col1, col, NULL, FLI_SLIDER_KNOB );
 }
 
 
@@ -185,9 +185,9 @@ draw_slider( FL_OBJECT * ob )
         else
             sprintf( valstr, "%.*f", sp->prec, sp->val );
 
-        fl_drw_box( ob->boxtype, bx, by, bw, bh, ob->col1, ob->bw );
-        fl_drw_text_beside( FL_ALIGN_CENTER, bx, by, bw, bh,
-                            ob->lcol, ob->lstyle, ob->lsize, valstr );
+        fl_draw_box( ob->boxtype, bx, by, bw, bh, ob->col1, ob->bw );
+        fl_draw_text_beside( FL_ALIGN_CENTER, bx, by, bw, bh,
+                             ob->lcol, ob->lstyle, ob->lsize, valstr );
     }
 
     if (    ( sp->draw_type == SLIDER_MOTION || sp->draw_type == SLIDER_JUMP )
@@ -201,12 +201,12 @@ draw_slider( FL_OBJECT * ob )
 
     if ( fl_is_center_lalign( ob->align ) )
     {
-        fli_drw_slider( ob, ob->col1, ob->col2,
-                        IS_FILL( ob ) ? NULL : ob->label,
-                        FLI_SLIDER_ALL & ~sp->mouse );
+        fli_draw_slider( ob, ob->col1, ob->col2,
+                         IS_FILL( ob ) ? NULL : ob->label,
+                         FLI_SLIDER_ALL & ~sp->mouse );
         
         /* Added 10/21/00 TCZ: need this to get the inside label right
-           otherwise fli_drw_slider() draws the lable centered on the
+           otherwise fli_draw_slider() draws the lable centered on the
            xfilled part!*/
 
         if ( IS_FILL( ob ) )
@@ -214,8 +214,8 @@ draw_slider( FL_OBJECT * ob )
     }
     else
     {
-        fli_drw_slider( ob, ob->col1, ob->col2, "",
-                        FLI_SLIDER_ALL & ~sp->mouse );
+        fli_draw_slider( ob, ob->col1, ob->col2, "",
+                         FLI_SLIDER_ALL & ~sp->mouse );
 
         if ( fl_is_inside_lalign( ob->align ) )
             fl_draw_object_label( ob );
@@ -224,8 +224,8 @@ draw_slider( FL_OBJECT * ob )
     }
 
     if ( sp->mouse != FLI_SLIDER_NONE )
-        fli_drw_slider( ob, ob->col1, sp->mouse ? FL_MCOL : ob->col2,
-                        "", sp->mouse );
+        fli_draw_slider( ob, ob->col1, sp->mouse ? FL_MCOL : ob->col2,
+                         "", sp->mouse );
 }
 
 
@@ -419,7 +419,7 @@ handle_mouse( FL_OBJECT    * obj,
         else
             return FL_RETURN_NONE;
     }
-    else if ( sp->react_to[ key - 1 ] )
+    else if ( REACT_TO( obj, key ) )
         newval = get_newvalue( obj, mx, my );
     else
         return FL_RETURN_NONE;
@@ -491,7 +491,7 @@ handle_motion( FL_OBJECT * obj,
     /* For non-scrollbar objects we're going to update the sliders position -
        if a shift key is pressed we here fake a smaller mouse movement */
 
-    if ( ! IS_SCROLLBAR( obj ) && sp->react_to[ key - 1 ] )
+    if ( ! IS_SCROLLBAR( obj ) && key <= FL_MBUTTON3 && REACT_TO( obj, key ) )
     {
         if ( shiftkey_down( ( ( XEvent * ) ev )->xmotion.state ) )
         {
@@ -503,9 +503,9 @@ handle_motion( FL_OBJECT * obj,
             }
             
             if ( IS_HSLIDER( obj ) )
-                mx = sp->old_mx + ( mx - sp->old_mx ) * FL_SLIDER_FINE;
+                mx = sp->old_mx + ( mx - sp->old_mx ) * FLI_SLIDER_FINE;
             else
-                my = sp->old_my + ( my - sp->old_my ) * FL_SLIDER_FINE;
+                my = sp->old_my + ( my - sp->old_my ) * FLI_SLIDER_FINE;
         }
         else
             sp->was_shift = 0;
@@ -721,7 +721,7 @@ handle_slider( FL_OBJECT * ob,
             {
                 ob->align = fl_to_outside_lalign( ob->align );
                 if ( fl_is_center_lalign( ob->align ) )
-                    ob->align = FL_SLIDER_ALIGN;
+                    ob->align = FLI_SLIDER_ALIGN;
             }
             compute_bounds( ob );
             break;
@@ -753,21 +753,33 @@ handle_slider( FL_OBJECT * ob,
             break;
 
         case FL_PUSH :
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             if ( ! ( ob->type & FL_VERT_PROGRESS_BAR ) )
                 ret |= handle_push( ob, mx, my, key, ev );
             break;
 
         case FL_MOTION :
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             if ( ! ( ob->type & FL_VERT_PROGRESS_BAR ) )
                 ret |= handle_motion( ob, mx, my, key, ev );
             break;
 
         case FL_UPDATE :
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             if ( ! ( ob->type & FL_VERT_PROGRESS_BAR ) )
                 ret |= handle_update( ob, mx, my, key );
             break;
 
         case FL_RELEASE :
+            if ( ! REACT_TO( ob, key ) )
+                break;
+
             if ( ! ( ob->type & FL_VERT_PROGRESS_BAR ) )
                 ret |= handle_release( ob, mx, my, key, ev );
             break;
@@ -790,18 +802,20 @@ create_slider( int          objclass,
                FL_Coord     h,
                const char * label )
 {
-    FL_OBJECT *ob;
+    FL_OBJECT *obj = fl_make_object( objclass, type, x, y, w, h, label,
+                                     handle_slider );
     FLI_SLIDER_SPEC *sp;
-    int i;
 
-    ob = fl_make_object( objclass, type, x, y, w, h, label, handle_slider );
-    ob->boxtype = FL_SLIDER_BOXTYPE;
-    ob->col1    = FL_SLIDER_COL1;
-    ob->col2    = FL_SLIDER_COL2;
-    ob->align   = FL_SLIDER_ALIGN;
-    ob->lcol    = FL_SLIDER_LCOL;
-    ob->lsize   = FL_TINY_SIZE;
-    ob->spec    = sp = fl_calloc( 1, sizeof *sp );
+    obj->boxtype      = FLI_SLIDER_BOXTYPE;
+    obj->col1         = FLI_SLIDER_COL1;
+    obj->col2         = FLI_SLIDER_COL2;
+    obj->align        = FLI_SLIDER_ALIGN;
+    obj->lcol         = FLI_SLIDER_LCOL;
+    obj->lsize        = FL_TINY_SIZE;
+    obj->set_react_to = fl_set_slider_mouse_buttons;
+
+    obj->spec = sp = fl_calloc( 1, sizeof *sp );
+
     sp->min            = 0.0;
     sp->max            = 1.0;
     sp->val            = sp->start_val = 0.5;
@@ -814,21 +828,19 @@ create_slider( int          objclass,
     sp->was_shift      = 0;
     sp->cross_over     = 0;
     sp->old_mx = sp->old_my = 0;
-    if ( IS_SCROLLBAR( ob ) )
+    if ( IS_SCROLLBAR( obj ) )
         sp->slsize    *= 1.5;
 
     sp->ldelta         = 0.1;
     sp->rdelta         = 0.05;
 
-    fl_set_object_dblbuffer( ob, 1 );
+    fl_set_object_dblbuffer( obj, 1 );
 
     /* Per default a slider reacts to the left mouse button only */
 
-    sp->react_to[ 0 ] = 1;
-    for ( i = 1; i < 3; i++ )
-        sp->react_to[ i ] = 0;
+    fl_set_object_mouse_buttons( obj, 1U );
 
-    return ob;
+    return obj;
 }
 
 
@@ -1184,15 +1196,11 @@ fl_set_slider_repeat( FL_OBJECT * ob,
  * buttons the slider object will react.
  ***************************************/
 
-void
+unsigned int
 fl_set_slider_mouse_buttons( FL_OBJECT    * obj,
                              unsigned int   mouse_buttons )
 {
-    FLI_SLIDER_SPEC *sp = obj->spec;
-    unsigned int i;
-
-    for ( i = 0; i < 3; i++, mouse_buttons >>= 1 )
-        sp->react_to[ i ] = mouse_buttons & 1;
+    return obj->react_to = mouse_buttons & 0x1F;
 }
 
 
@@ -1205,24 +1213,7 @@ void
 fl_get_slider_mouse_buttons( FL_OBJECT    * obj,
                              unsigned int * mouse_buttons )
 {
-    FLI_SLIDER_SPEC *sp;
-    int i;
-    unsigned int k;
-
-    if ( ! obj )
-    {
-        M_err( "fl_get_slider_mouse_buttons", "NULL object" );
-        return;
-    }
-
-    if ( ! mouse_buttons )
-        return;
-
-    sp = obj->spec;
-
-    *mouse_buttons = 0;
-    for ( i = 0, k = 1; i < 3; i++, k <<= 1 )
-        *mouse_buttons |= sp->react_to[ i ] ? k : 0;
+    *mouse_buttons = fl_get_object_mouse_buttons( obj );
 }
 
 
