@@ -45,8 +45,6 @@ static int handle_tbox( FL_OBJECT *,
                         void * );
 
 static GC create_gc( FL_OBJECT *,
-                     int,
-                     int,
                      FL_COLOR,
                      int,
                      int,
@@ -319,7 +317,7 @@ fli_tbox_insert_line( FL_OBJECT  * obj,
     tl->specialGC     = None;
     tl->incomp_esc    = 0;
 
-    /* Check for flags at the start of the line. When we're done 'p'
+    /* Check for flags at the start of the line. When we're done 'p' will
        points to the start of the string to be shown in the textbox. */
 
     while ( *p && *p == sp->specialkey && ! done )
@@ -608,16 +606,16 @@ fli_tbox_add_chars( FL_OBJECT  * obj,
        of the line from the old text and the new text, delete te old line and
        then draw a new one in its place.
 
-       Another question is how to deal with situations where by a previous
-       call with e.g. "@C3" a color was selected and in the next call the
+       Another question is how to deal with situations where, by a previous
+       call with e.g. "@C3", a color was selected and in the next call the
        string to be added starts with a digit, let's say '7'. One possibility
        would be to re-evaluate the combined string to mean "@C37", setting
        a different color. But since in older versions this didn't happen
-       (and some users may rely on this, it is assumed that this isn't the
+       (and some users may rely on this), it is assumed that this isn't the
        users intention and to avoid the two digits to become collated and
        interpreted as the number of a different color "@ " is inserted in
        between the digits, with "@ " treated as a separator between digits
-       (the intended ose of "@ " is to allow lines like "@C3@ 2. Chapter", i.
+       (the intended use of "@ " is to allow lines like "@C3@ 2. Chapter", i.
        e. having a color specification, followed by printable text that starts
        with a digit). */
 
@@ -669,7 +667,7 @@ fli_tbox_add_chars( FL_OBJECT  * obj,
     strcpy( tl->fulltext, old_fulltext );
     strcat( tl->fulltext, new_text );
     tl->text = tl->fulltext + ( old_text - old_fulltext );
-    tl->len = strlen( tl->text ); //new_len;
+    tl->len = strlen( tl->text );
 
     fli_safe_free( old_fulltext );
 
@@ -772,6 +770,7 @@ fli_tbox_clear( FL_OBJECT * obj )
             XFreeGC( flx->display, sp->lines[ i ]->specialGC );
             sp->lines[ i ]->specialGC = None;
         }
+
         fli_safe_free( sp->lines[ i ]->fulltext );
         fli_safe_free( sp->lines[ i ] );
     }
@@ -934,6 +933,17 @@ fli_tbox_set_fontsize( FL_OBJECT * obj,
 }
 
 
+/*************************************
+ * Returns the default font size
+ *************************************/
+
+int
+fli_tbox_get_fontsize( FL_OBJECT * obj )
+{
+    return ( ( FLI_TBOX_SPEC * ) obj->spec )->def_size;
+}
+
+
 /**************************************
  * Sets a new font style for all lines
  * drawn with default settings
@@ -1017,6 +1027,17 @@ fli_tbox_set_fontstyle( FL_OBJECT * obj,
 }
 
 
+/*************************************
+ * Returns the default font style
+ *************************************/
+
+int
+fli_tbox_get_fontstyle( FL_OBJECT * obj )
+{
+    return ( ( FLI_TBOX_SPEC * ) obj->spec )->def_style;
+}
+
+
 /*************************************
  * Sets the x-offset in pixels of the
  * text displayed in the textbox
@@ -1401,9 +1422,12 @@ fli_tbox_make_line_selectable( FL_OBJECT * obj,
             }
 
             if ( FL_ObjWin( obj ) )
-                tl->specialGC = create_gc( obj, tl->style, tl->size,
+            {
+                tl->specialGC = create_gc( obj,
                                            state ? obj->lcol : FL_INACTIVE,
                                            sp->x, sp->y, sp->w, sp->h );
+                tl->special_color = state ? obj->lcol : FL_INACTIVE;
+            }
         }
     }
 
@@ -1456,8 +1480,6 @@ fli_tbox_set_dblclick_callback( FL_OBJECT      * obj,
 
 static GC
 create_gc( FL_OBJECT * obj,
-           int         style,
-           int         size,
            FL_COLOR    color,
            int         clip_x,
            int         clip_y,
@@ -1483,13 +1505,6 @@ create_gc( FL_OBJECT * obj,
     xgcv.foreground = fl_get_flcolor( color );
     gc = XCreateGC( flx->display, FL_ObjWin( obj ), gcvm, &xgcv );
 
-    if ( size > 0 && style >= 0 )
-    {
-        XFontStruct *xfs = fl_get_fntstruct( style, size );
-
-        XSetFont( flx->display, gc, xfs->fid );
-    }
-
     fl_set_gc_clipping( gc, obj->x + clip_x, obj->y + clip_y, clip_w, clip_h );
 
     return gc;
@@ -1565,15 +1580,16 @@ fli_tbox_prepare_drawing( FL_OBJECT * obj )
     if ( sp->defaultGC )
         XFreeGC( flx->display, sp->defaultGC );
 
-    sp->defaultGC = create_gc( obj, sp->def_style, sp->def_size, obj->lcol,
+    sp->defaultGC = create_gc( obj, obj->lcol,
                                sp->x, sp->y, sp->w, sp->h );
+    sp->default_color = obj->lcol;
 
     /* Create background GC for redraw deselected lines */
 
     if ( sp->backgroundGC )
         XFreeGC( flx->display, sp->backgroundGC );
 
-    sp->backgroundGC = create_gc( obj, -1, 0, obj->col1,
+    sp->backgroundGC = create_gc( obj, obj->col1,
                                   sp->x - ( LEFT_MARGIN > 0 ),
                                   sp->y, sp->w + ( LEFT_MARGIN > 0 ), sp->h );
 
@@ -1582,7 +1598,7 @@ fli_tbox_prepare_drawing( FL_OBJECT * obj )
     if ( sp->selectGC )
         XFreeGC( flx->display, sp->selectGC );
 
-    sp->selectGC = create_gc( obj, -1, 0,
+    sp->selectGC = create_gc( obj,
                               fli_dithered( fl_vmode ) ?
                               FL_BLACK : obj->col2,
                               sp->x - ( LEFT_MARGIN > 0 ), sp->y,
@@ -1593,8 +1609,9 @@ fli_tbox_prepare_drawing( FL_OBJECT * obj )
     if ( sp->nonselectGC )
         XFreeGC( flx->display, sp->nonselectGC );
 
-    sp->nonselectGC = create_gc( obj, sp->def_style, sp->def_size, FL_INACTIVE,
+    sp->nonselectGC = create_gc( obj, FL_INACTIVE,
                                  sp->x, sp->y, sp->w, sp->h );
+    sp->nonselect_color = FL_INACTIVE;
 
     /* Special GC for text of selected lines in B&W */
 
@@ -1603,9 +1620,10 @@ fli_tbox_prepare_drawing( FL_OBJECT * obj )
         if ( sp->bw_selectGC )
             XFreeGC( flx->display, sp->bw_selectGC );
 
-        sp->bw_selectGC = create_gc( obj, sp->def_style, sp->def_size, FL_WHITE,
+        sp->bw_selectGC = create_gc( obj, FL_WHITE,
                                      sp->x - ( LEFT_MARGIN > 0 ), sp->y,
                                      sp->w + ( LEFT_MARGIN > 0 ), sp->h );
+        sp->bw_select_color = FL_WHITE;
     }
 
     /* Lines with non-default fonts or colors have their own GCs */
@@ -1623,8 +1641,9 @@ fli_tbox_prepare_drawing( FL_OBJECT * obj )
             tl->specialGC = None;
         }
 
-        tl->specialGC = create_gc( obj, tl->style, tl->size, tl->color,
+        tl->specialGC = create_gc( obj, tl->color,
                                    sp->x, sp->y, sp->w, sp->h );
+        tl->special_color = tl->color;
     }
 
     sp->no_redraw = 1;
@@ -1687,12 +1706,6 @@ draw_tbox( FL_OBJECT * obj )
     fl_draw_box( obj->boxtype, obj->x, obj->y, obj->w, obj->h,
                  obj->col1, obj->bw );
 
-    XFillRectangle( flx->display, FL_ObjWin( obj ),
-                    sp->backgroundGC,
-                    obj->x + sp->x - ( LEFT_MARGIN > 0 ),
-                    obj->y + sp->y + sp->w - sp->yoffset,
-                    sp->w + ( LEFT_MARGIN > 0 ), sp->h );
-
     if ( sp->num_lines == 0 )
         return;
 
@@ -1702,6 +1715,7 @@ draw_tbox( FL_OBJECT * obj )
     {
         TBOX_LINE *tl;
         GC activeGC = sp->defaultGC;
+        FL_COLOR active_color = sp->default_color;
 
         tl = sp->lines[ i ];
 
@@ -1729,11 +1743,13 @@ draw_tbox( FL_OBJECT * obj )
         /* Draw background of line in selection color if necessary*/
 
         if ( tl->selected )
+        {
+            fli_apply_clipping_to_gc( sp->selectGC );
             XFillRectangle( flx->display, FL_ObjWin( obj ), sp->selectGC,
                             obj->x + sp->x - ( LEFT_MARGIN > 0 ),
                             obj->y + sp->y + tl->y - sp->yoffset,
                             sp->w + ( LEFT_MARGIN > 0 ), tl->h );
-
+        }
 
         /* If there's no text or the text isn't visible within the textbox
            nothing needs to be drawn */
@@ -1747,17 +1763,24 @@ draw_tbox( FL_OBJECT * obj )
            a special GC just for that line */
 
         if ( ! tl->selectable )
+        {
             activeGC = sp->nonselectGC;
+            active_color = sp->nonselect_color;
+        }
 
         if ( tl->is_special )
         {
             if ( ! tl->specialGC )
-                tl->specialGC = create_gc( obj, tl->style, tl->size,
+            {
+                tl->specialGC = create_gc( obj,
                                            tl->selectable ?
                                            tl->color : FL_INACTIVE,
                                            sp->x, sp->y, sp->w, sp->h );
+                tl->special_color = tl->selectable ? tl->color : FL_INACTIVE;
+            }
 
             activeGC = tl->specialGC;
+            active_color = tl->special_color;
         }
 
         /* Set up GC for selected lines in B&W each time round - a bit slow,
@@ -1765,12 +1788,11 @@ draw_tbox( FL_OBJECT * obj )
 
         if ( fli_dithered( fl_vmode ) && tl->selected )
         {
-            XFontStruct *xfs = fl_get_fntstruct( tl->style, tl->size );
-            
-            XSetFont( flx->display, sp->bw_selectGC, xfs->fid );
             XSetForeground( flx->display, sp->bw_selectGC,
                             fl_get_flcolor( FL_WHITE ) );
+            sp->bw_select_color = FL_WHITE;
             activeGC = sp->bw_selectGC;
+            active_color = sp->bw_select_color;
         }
 
         /* Now draw the line, underlined if necessary */
@@ -1782,10 +1804,22 @@ draw_tbox( FL_OBJECT * obj )
                          ( fli_dithered( fl_vmode ) && tl->selected ) ?
                          FL_WHITE : tl->color );
 
-        fli_draw_stringTAB( FL_ObjWin( obj ), activeGC,
+        if ( activeGC != sp->bw_selectGC )
+            fl_set_text_clipping( obj->x + sp->x,
+                                  obj->y + sp->y,
+                                  sp->w, sp->h );
+        else
+            fl_set_text_clipping( obj->x + sp->x - ( LEFT_MARGIN > 0 ),
+                                  obj->y + sp->y,
+                                  sp->w + ( LEFT_MARGIN > 0 ), sp->h );
+
+
+        fli_draw_stringTAB( FL_ObjWin( obj ), active_color,
                             obj->x + sp->x - sp->xoffset + tl->x,
                             obj->y + sp->y - sp->yoffset + tl->y + tl->asc,
                             tl->style, tl->size, tl->text, tl->len, 0 );
+
+        fl_unset_text_clipping( );
     }
 
     fl_unset_clipping( );
@@ -1864,7 +1898,7 @@ fli_tbox_get_topline( FL_OBJECT * obj )
         return -1;
 
     /* If the box was never shown assume that the very first line will
-       be the one that's goin tp be in topmost position */
+       be the one that's going to be in topmost position */
 
     if ( ! sp->def_height )
         return 0;
