@@ -81,6 +81,8 @@ typedef struct
     char           * title;         /* Menu title                   */
     Window           win;           /* menu window                  */
     Cursor           cursor;        /* cursor for the pup           */
+    FL_COLOR         active_color;
+    FL_COLOR         inactive_color;
     GC               gc_active;     /* GC for main text             */
     GC               gc_inactive;   /* GC for inactive text         */
     MenuItem       * item[ FL_MAXPUPI + 1 ];
@@ -139,10 +141,20 @@ static int pup_bw_is_set = 0;
 
 static PopUP *menu_rec = NULL;
 
+static void reset_max_width( PopUP * m );
+
+#if FL_ENABLE_XFT
+static XftFont *pup_font_struct = NULL;        /* popup main text font */
+#else
 static XFontStruct *pup_font_struct = NULL;        /* popup main text font */
+#endif
 static int pup_ascent = 0,                         /* font properties */
            pup_desc = 0;
+#if FL_ENABLE_XFT
+static XftFont *pup_title_font_struct = NULL;  /* popup title text font */
+#else
 static XFontStruct *pup_title_font_struct = NULL;  /* popup title text font */
+#endif
 static int pup_title_ascent = 0,
            pup_title_desc   = 0;
 static Cursor pup_defcursor = 0;
@@ -159,25 +171,33 @@ static int pup_internal_showpup_call = 0;
  ***************************************/
 
 static void
-init_pupfont( void )
+init_pupfont( PopUP * m )
 {
+#if ! FL_ENABLE_XFT        
     XCharStruct chs;
     int junk;
+#endif
 
-    if ( ! pup_title_font_struct )
-    {
-        pup_title_font_struct = fl_get_fntstruct( pup_title_font_style,
-                                                  pup_title_font_size );
-        XTextExtents( pup_title_font_struct, "qjQb", 4, &junk,
-                      &pup_title_ascent, &pup_title_desc, &chs );
-    }
+    pup_title_font_struct = fl_get_fntstruct( pup_title_font_style,
+                                              pup_title_font_size );
+#if FL_ENABLE_XFT        
+    pup_title_ascent = pup_title_font_struct->ascent;
+    pup_title_desc   = pup_title_font_struct->descent;
+#else
+    XTextExtents( pup_title_font_struct, "qjQb", 4, &junk,
+                  &pup_title_ascent, &pup_title_desc, &chs );
+#endif
 
-    if ( ! pup_font_struct )
-    {
-        pup_font_struct = fl_get_fntstruct( pup_font_style, pup_font_size );
-        XTextExtents( pup_font_struct, "qjQb", 4, &junk, &pup_ascent,
-                      &pup_desc, &chs );
-    }
+    pup_font_struct = fl_get_fntstruct( pup_font_style, pup_font_size );
+#if FL_ENABLE_XFT        
+    pup_ascent = pup_font_struct->ascent;
+    pup_desc   = pup_font_struct->descent;
+#else
+    XTextExtents( pup_font_struct, "qjQb", 4, &junk, &pup_ascent,
+                  &pup_desc, &chs );
+#endif
+
+    reset_max_width( m );
 }
 
 
@@ -195,6 +215,7 @@ init_pup( PopUP * m )
     m->title_width  = 0;
     m->win          = None;
     m->gc_active    = m->gc_inactive = None;
+    m->active_color = m->inactive_color = FL_NOCOLOR;
     m->bw           = pup_bw;
     m->title        = NULL;
     m->item[ 0 ]    = NULL;
@@ -203,8 +224,7 @@ init_pup( PopUP * m )
         pup_defcursor = fli_get_cursor_byname( XC_sb_right_arrow );
     m->cursor       = pup_defcursor;
     m->lpad         = m->rpad = PADW;
-    init_pupfont( );
-    m->cellh        = pup_ascent + pup_desc + 2 * m->padh;
+    init_pupfont( m );
     m->isEntry      = 0;
     m->form         = NULL;
 }
@@ -274,7 +294,8 @@ reset_max_width( PopUP * m )
         b = t = fl_strdup( m->title );
         while ( ( b = strchr( b, '\b' ) ) )
             memmove( b, b + 1, strlen( b ) );
-        m->title_width = XTextWidth( pup_title_font_struct, t, strlen( t ) );
+        m->title_width = fli_get_string_width( pup_title_font_struct,
+                                               t, strlen( t ) );
         fl_free( t );
     }
     else
@@ -472,35 +493,14 @@ parse_entry( int          n,
 
         if ( flags & M_TITLE )
         {
-            char *t,
-                 *b;
-
             m->title = fl_strdup( c );
-            b = t = fl_strdup( c );
-            while ( ( b = strchr( b, '\b' ) ) )
-                memmove( b, b + 1, strlen( b ) );
-            m->title_width = XTextWidth( pup_title_font_struct,
-                                         t, strlen( t ) );
-            fl_free( t );
             fl_free( item );
             m->item[ m->nitems ] = NULL;
         }
         else
         {
-            char *t,
-                 *b;
-
             item->str = fl_strdup( c );
             item->len = strlen( item->str );
-
-            b = t = fl_strdup( item->str );
-            while ( ( b = strchr( b, '\b' ) ) )
-                memmove( b, b + 1, strlen( b ) );
-            m->maxw = FL_max( m->maxw,
-                              fl_get_string_widthTAB( pup_font_style,
-                                                      pup_font_size,
-                                                      t, strlen( t ) ) );
-            fl_free( t );
             m->nitems++;
         }
     }
@@ -522,6 +522,7 @@ close_pupwin( PopUP * pup )
 {
     if ( pup->win )
     {
+        fl_winset( None );
         XDestroyWindow( flx->display, pup->win );
         wait_for_close( pup->win );
         pup->win = None;
@@ -548,14 +549,15 @@ fli_init_pup( void )
 
     for ( mr = menu_rec; mr < menu_rec + fl_maxpup; mr++ )
     {
-        mr->used       = 0;
-        mr->title      = NULL;
-        mr->win        = None;
-        mr->cursor     = None;
-        mr->gc_active  = mr->gc_inactive = None;
-        mr->menu_cb    = NULL;
-        mr->enter_cb   = mr->leave_cb = NULL;
-        mr->enter_data = mr->leave_data = NULL;
+        mr->used         = 0;
+        mr->title        = NULL;
+        mr->win          = None;
+        mr->cursor       = None;
+        mr->gc_active    = mr->gc_inactive = None;
+        mr->active_color = mr->inactive_color = FL_NOCOLOR;
+        mr->menu_cb      = NULL;
+        mr->enter_cb     = mr->leave_cb = NULL;
+        mr->enter_data   = mr->leave_data = NULL;
 
         for ( i = 0; i <= FL_MAXPUPI; i++ )
             mr->item[ i ] = NULL;
@@ -572,7 +574,6 @@ fli_init_pup( void )
 int
 fl_setpup_default_fontsize( int size )
 {
-    int i;
     int old_pup_font_size = pup_font_size;
 
     if ( size <= 0 )
@@ -583,19 +584,11 @@ fl_setpup_default_fontsize( int size )
     pup_font_size = size;
     pup_title_font_size = size;
 
-    pup_font_struct = pup_title_font_struct = 0;
+    pup_font_struct = pup_title_font_struct = NULL;
 
     if ( ! flx->display )
         return old_pup_font_size;
 
-    init_pupfont( );
-
-    for ( i = 0; i < fl_maxpup; i++ )
-    {
-        reset_max_width( menu_rec + i );
-        close_pupwin( menu_rec + i );
-    }
-
     return old_pup_font_size;
 }
 
@@ -606,7 +599,6 @@ fl_setpup_default_fontsize( int size )
 int
 fl_setpup_default_fontstyle( int style )
 {
-    int i;
     int old_pup_font_style = pup_font_style;
 
     if ( ! flx->display )
@@ -621,11 +613,6 @@ fl_setpup_default_fontstyle( int style )
     pup_title_font_style = style;
     pup_font_struct      = pup_title_font_struct = NULL;
 
-    init_pupfont( );
-
-    for ( i = 0; i < fl_maxpup; i++ )
-        reset_max_width( menu_rec + i );
-
     return old_pup_font_style;
 }
 
@@ -1232,6 +1219,7 @@ pup_interact( PopUP * m )
 
 
 /***************************************
+ * Grab both pointer and keyboard
  ***************************************/
 
 static void
@@ -1289,6 +1277,8 @@ fl_dopup( int n )
 
     pup_subreturn = -1;
 
+    init_pupfont( m );
+
     pup_level++;
     pup_internal_showpup_call = 1;
     fl_showpup( n );
@@ -1579,7 +1569,8 @@ fl_setpup_title( int          nm,
     b = t = fl_strdup( title ? title : "" );
     while ( ( b = strchr( b, '\b' ) ) )
         memmove( b, b + 1, strlen( b ) );
-    m->title_width = XTextWidth( pup_title_font_struct, t, strlen( t ) );
+    m->title_width = fli_get_string_width( pup_title_font_struct,
+                                           t, strlen( t ) );
     fl_free( t );
 }
 
@@ -1793,12 +1784,14 @@ draw_item( PopUP * m,
     char *str;
     MenuItem *item;
     GC gc;
+    FL_COLOR color;
 
     if ( j < 0 || j >= m->nitems )
         return;
 
     item = m->item[ j ];
     gc = ( item->mode & FL_PUP_GREY ) ? m->gc_inactive : m->gc_active;
+    color = ( item->mode & FL_PUP_GREY ) ? m->inactive_color : m->active_color;
     str = item->str;
 
     if ( ! ( item->mode & FL_PUP_GREY ) )
@@ -1830,7 +1823,8 @@ draw_item( PopUP * m,
     /* show text */
 
     j = str[ 0 ] == '\010';
-    fli_draw_stringTAB( m->win, gc,
+
+    fli_draw_stringTAB( m->win, color,
                         m->lpad + 2 * bw, y + m->padh + pup_ascent,
                         pup_font_style, pup_font_size, str + j,
                         strlen( str ) - j, 0 );
@@ -1885,16 +1879,45 @@ draw_title( Display  * d,
 
     fl_set_font( pup_title_font_style, pup_title_font_size );
     fli_textcolor( pup_text_color );
+#if FL_ENABLE_XFT
+    d = d;
+
+    XftDrawChange( flx->textdraw, w );
+
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x - 1, y - 1,
+                    ( XftChar8 const * ) t, n );
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x,     y - 1,
+                    ( XftChar8 const * ) t, n );
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x + 1, y - 1,
+                    ( XftChar8 const * ) t, n );
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x - 1, y,
+                    ( XftChar8 const * ) t, n );
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x + 1, y,
+                    ( XftChar8 const * ) t, n );
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x - 1, y + 1,
+                    ( XftChar8 const * ) t, n );
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x, y + 1,
+                    ( XftChar8 const * ) t, n );
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x + 1, y + 1,
+                    ( XftChar8 const * ) t, n );
+#else
     XDrawString( d, w, flx->textgc, x - 1, y - 1, t, n );
-    XDrawString( d, w, flx->textgc, x, y - 1, t, n );
+    XDrawString( d, w, flx->textgc, x,     y - 1, t, n );
     XDrawString( d, w, flx->textgc, x + 1, y - 1, t, n );
-    XDrawString( d, w, flx->textgc, x - 1, y, t, n );
-    XDrawString( d, w, flx->textgc, x + 1, y, t, n );
+    XDrawString( d, w, flx->textgc, x - 1, y,     t, n );
+    XDrawString( d, w, flx->textgc, x + 1, y,     t, n );
     XDrawString( d, w, flx->textgc, x - 1, y + 1, t, n );
-    XDrawString( d, w, flx->textgc, x, y + 1, t, n );
+    XDrawString( d, w, flx->textgc, x, y + 1,     t, n );
     XDrawString( d, w, flx->textgc, x + 1, y + 1, t, n );
+#endif
+
     fli_textcolor( FL_WHITE );
+#if FL_ENABLE_XFT
+    XftDrawString8( flx->textdraw, &flx->textcolor, flx->fs, x, y,
+                    ( XftChar8 const * ) t, n );
+#else
     XDrawString( d, w, flx->textgc, x, y, t, n );
+#endif
 
     fl_free( t );
 }
@@ -2090,23 +2113,26 @@ fl_showpup( int n )
             XGCValues xgcv;
 
             xgcv.foreground     = fl_get_flcolor( pup_text_color );
-            xgcv.font           = pup_font_struct->fid;
             xgcv.stipple        = FLI_INACTIVE_PATTERN;
-            vmask               = GCForeground | GCFont | GCStipple;
+            vmask               = GCForeground | GCStipple;
 
             /* GC for main text */
 
             m->gc_active = XCreateGC( flx->display, m->win, vmask, &xgcv );
 
+            m->active_color = pup_text_color;
+
             /* GC for inactive text */
 
             xgcv.foreground = fl_get_flcolor( FL_INACTIVE );
             m->gc_inactive = XCreateGC( flx->display, m->win, vmask, &xgcv );
 
-            /* special hack for B&W */
+            /* Special hack for B&W */
 
             if ( fli_dithered( fl_vmode ) )
                 XSetFillStyle( flx->display, m->gc_inactive, FillStippled );
+
+            m->inactive_color = FL_INACTIVE;
         }
 
         XSetWMColormapWindows( flx->display, fl_root, &m->win, 1 );
@@ -2244,6 +2270,7 @@ fl_setpup_maxpup( int n )
         menu_rec[ i ].cursor = None;
 
         menu_rec[ i ].gc_active = menu_rec[ i ].gc_inactive = None;
+        menu_rec[ i ].active_color = menu_rec[ i ].inactive_color = FL_NOCOLOR;
 
         for ( j = 0; j <= FL_MAXPUPI; j++ )
             menu_rec[ i ].item[ j ] = NULL;
