@@ -53,6 +53,28 @@ static void do_underline_all( FL_Coord,
                               unsigned long *,
                               unsigned long * );
 
+#if FL_ENABLE_XFT
+static void
+draw_string( Display        * display,
+             XftFont        * font,
+             FL_COLOR         fg_color,
+             int              x,
+             int              y,
+             const XftChar8 * str,
+             size_t           len,
+             int              draw_background );
+#else
+/* type fitting both XDrawString() and XDrawImageString() */
+
+typedef int ( * DrawString )( Display    * display,
+                              Drawable     d,
+                              GC           gc,
+                              int          x,
+                              int          y,
+                              const char * string,
+                              int          length );
+#endif
+
 #define NUM_LINES_INCREMENT  64
 
 static struct LINE_INFO {
@@ -103,30 +125,22 @@ fli_get_max_pixels_line( void )
 }
 
 
-typedef int ( * DrawString )( Display *,
-                              Drawable, GC,
-                              int,
-                              int,
-                              const char *,
-                              int );
-
-
 /***************************************
  * Major text drawing routine. It draws text (possibly consisting of several
  * lines) into the box specified via the coordinates and using the given
  * alignment relative to that box. Also a cursor is drawn. For lines that
- * contain a specal character indicating underlining this is also handled.
+ * contain a special character indicating underlining this is also handled.
  * Finally, parts of the text can be shown as selected.
  *
  * Arguments:
  *  align       Alignment of the text relative to the box
  *  x, ym w, h  postion and size of box
- *  clip        0 = no clipping is to be used at all
- *              1 = clipping is to be done by this function
+ *  clip         0 = no clipping is to be used at all
+ *               1 = clipping is to be done by this function
  *              -1 = clipping is done but already has been set externally
  *  backcol     color to be used for selected text
  *  forecol     color for (unselected) text
- *  curscol     color of cursor
+ *  curscol     color for cursor
  *  style       text font style
  *  size        text font size
  *  curspos     index into string where cursor is to be drawn (if negative or
@@ -135,15 +149,15 @@ typedef int ( * DrawString )( Display *,
  *  selend      index into string where selection ends (to get selection this
  *              must be larger than 'selstart')
  *  istr        pointer to the text to be drawn
- *  img         if non-zero use XDrawImageString() instead of XDrawString()
- *              to draw the text
+ *  img         if non-zero also draw background (use XDrawImageString()
+ *              instead of XDrawString() to draw the text)
  *  topline     first line of the text to be actually shown (counting starts
  *              at 1)
  *  endline     last line to be shown (if less than 1 or too large is replaced
  *              by the number of lines in the text)
- *  bkcol       background color
+ *  bkcol       background color (used when 'img' is true)
  *
- * Returns the width (in pixel) of the widest line of the text.
+ * Returns the width (in pixel) of the widest line of the text drawn.
  ***************************************/
 
 int
@@ -160,7 +174,7 @@ fli_draw_string( int           align,
                  int           size,
                  int           curspos,
                  int           selstart,
-                int           selend,
+                 int           selend,
                  const char *  istr,
                  int           img,
                  int           topline,
@@ -174,19 +188,23 @@ fli_draw_string( int           align,
         vertalign;
     char * str = NULL,
          * p = NULL;
+#if FL_ENABLE_XFT
+    XftFont * font = fl_get_font_struct( style, size );
+#else
     DrawString drawIt = img ? XDrawImageString : XDrawString;
+#endif
 
     /* Check if anything has to be drawn at all - do nothing if we either
        have no window or the cursor is to be drawn somewhere else than in
        the very first position and there's no string to output. It would
        be tempting to also bail out if the height 'h' is 0 or even negative
        but there are some code paths were this actually may happen and we
-       wouldn't outut a string even though it is needed (I know, it's a
+       wouldn't output a string even though it is needed (I know, it's a
        bloody mess but fixing it right now would probably take a few weeks
        and even might break existing code...) */
 
     if (    flx->win == None
-         || ( curspos > 0 && ! ( istr && *istr ) ) )
+         || ( curspos > 0 && ( ! istr || ! *istr ) ) )
         return 0;
 
     /* We operate only on a copy of the input string */
@@ -232,7 +250,7 @@ fli_draw_string( int           align,
         endline = lnumb;
 
     /* Calculate coordinates of all lines (for y the baseline position),
-     for that make sure the correct font is set up */
+       for that make sure the correct font is set up */
 
     fl_set_font( style, size );
     fli_get_hv_align( align, &horalign, &vertalign );
@@ -270,7 +288,7 @@ fli_draw_string( int           align,
 
         /* Determine the width (in pixel) of the line) */
 
-        width = XTextWidth( flx->fs, line->str, line->len );
+        width = fli_get_text_width( line->str, line->len );
 
         if ( width > max_pixels )
         {
@@ -278,7 +296,7 @@ fli_draw_string( int           align,
             max_pixelline = i;
         }
 
-        /* Calculate the x- and y- positon of were to print the text */
+        /* Calculate the x- and y- positon of where to print the text */
 
         switch ( horalign )
         {
@@ -330,7 +348,7 @@ fli_draw_string( int           align,
     fli_textcolor( forecol );
     fli_bk_textcolor( bkcol );
 
-    /* Draw the lines requested */
+    /* Draw all the lines requested */
 
     for ( i = topline; i < endline; i++ )
     {
@@ -351,8 +369,13 @@ fli_draw_string( int           align,
 
         /* Draw the text */
 
+#if FL_ENABLE_XFT
+        draw_string( flx->display, font, forecol, line->x, line->y,
+                     ( const XftChar8 * ) line->str, line->len, img );
+#else
         drawIt( flx->display, flx->win, flx->textgc,
                 line->x, line->y, line->str, line->len );
+#endif
 
         /* Draw selection area if required - for this we need to draw
            the selection background and then redraw the text in this
@@ -388,9 +411,9 @@ fli_draw_string( int           align,
                region (the -1 in the calculation  of wsel is a fudge factor
                to make it look a bit better) */
 
-            xsel = line->x + XTextWidth( flx->fs, line->str, start );
+            xsel = line->x + fli_get_text_width( line->str, start );
 
-            wsel = XTextWidth( flx->fs, line->str + start, len ) - 1;
+            wsel = fli_get_text_width( line->str + start, len ) - 1;
             if ( xsel + wsel > x + w )
                 wsel = x + w - xsel;
 
@@ -399,9 +422,14 @@ fli_draw_string( int           align,
             fl_rectf( xsel, line->y - flx->fasc, wsel,
                       flx->fheight, forecol );
 
-            fli_textcolor( backcol );  /* bkcol ? */
+#if FL_ENABLE_XFT
+            draw_string( flx->display, font, backcol, xsel, line->y,
+                         ( const XftChar8 * ) ( line->str + start ), len, img );
+#else
+            fli_textcolor( backcol );
             drawIt( flx->display, flx->win, flx->textgc, xsel,
                     line->y, line->str + start, len );
+#endif
             fli_textcolor( forecol );
 
             if ( line->underline_index > 0 )
@@ -443,8 +471,7 @@ fli_draw_string( int           align,
         if (    curspos >= line->index
              && curspos <= line->index + line->len )
         {
-            int tt = XTextWidth( flx->fs, line->str,
-                                 curspos - line->index );
+            int tt = fli_get_text_width( line->str, curspos - line->index );
 
             fl_rectf( line->x + tt, line->y - flx->fasc,
                       2, flx->fheight, curscol );
@@ -476,9 +503,11 @@ fli_draw_string( int           align,
         fl_rectf( xc, yc, 2, flx->fheight, curscol );
     }
 
+    /* Free our copy of the string */
+
     fli_safe_free( str );
 
-    /* Reset clipping if required */
+    /* Reset clipping if we set it */
 
     if ( clip > 0 )
         fl_unset_text_clipping( );
@@ -488,6 +517,12 @@ fli_draw_string( int           align,
 
 
 /***************************************
+ * Function returns the index of the character in the label of the object
+ * the mouse is over or -1 if it's not over the label. Note that the function
+ * has some limitations: it can only be used on labels inside of the object
+ * and the label string may not contain underline characters (and the label
+ * can't be a symbol) - if you try to use it on labels that don't satisfy
+ * these requirements -1 is returned. 
  ***************************************/
 
 int
@@ -534,7 +569,7 @@ fl_get_label_char_at_mouse( FL_OBJECT * obj )
  * mouse is to the left of the start of the line)
  * The function expects a string that doesn't contain mon-printable characters
  * (except '\n' for starts a new lines)
- * This function is supposed to ork on text drawn using fli_draw_string()
+ * This function is supposed to work on text drawn using fli_draw_string()
  * using the same relevant arguments (alignment, box, font style and size
  * and string) as passed to this function.
  *
@@ -654,7 +689,7 @@ fli_get_pos_in_string( int          align,
 
     /* Calculate width and start x-coordinate of the line */
 
-    width = XTextWidth( flx->fs, line->str, line->len );
+    width = fli_get_text_width( line->str, line->len );
 
     switch ( horalign )
     {
@@ -699,7 +734,7 @@ fli_get_pos_in_string( int          align,
 
     *xp = ( double ) ( xpos * line->len ) / width;
 
-    xlen = XTextWidth( flx->fs, line->str, ++*xp );
+    xlen = fli_get_text_width( line->str, ++*xp );
 
     /* If we don't have hit it directly search to the left or right */
 
@@ -708,7 +743,7 @@ fli_get_pos_in_string( int          align,
         do
         {
             *xp -= 1;
-            xlen = XTextWidth( flx->fs, line->str, *xp );
+            xlen = fli_get_text_width( line->str, *xp );
         }
         while ( *xp > 0 && xlen > xpos );
         *xp += 1;
@@ -717,7 +752,7 @@ fli_get_pos_in_string( int          align,
         do
         {
             *xp += 1;
-            xlen = XTextWidth( flx->fs, line->str, *xp );
+            xlen = fli_get_text_width( line->str, *xp );
         }
         while ( *xp < lines->len && xlen < xpos );
 
@@ -732,58 +767,59 @@ fli_get_pos_in_string( int          align,
 ***/
 
 /***************************************
- * Draws a (multi-line) text with a cursor on a white background
- ***************************************/
-
-void
-fl_draw_text_cursor( int          align,
-                     FL_Coord     x,
-                     FL_Coord     y,
-                     FL_Coord     w,
-                     FL_Coord     h,
-                     FL_COLOR     c,
-                     int          style,
-                     int          size,
-                     const char * str,
-                     int          cc,
-                     int          pos )
-{
-    fli_draw_string( align, x, y, w, h, 0, FL_WHITE, c, cc,
-                     style, size, pos, 0, -1, str, 0, 0, 0, 0 );
-}
-
-
-/***************************************
+ * Draw text with cursor and, if 'bk' is set, also the background
+ * (but with no highlighting)
  ***************************************/
 
 static void
-fli_draw_text_cursor( int          align,
-                      FL_Coord     x,
+fli_draw_text_cursor( int          align,   /* alignment in box */
+                      FL_Coord     x,       /* bounding box geometry */
                       FL_Coord     y,
                       FL_Coord     w,
                       FL_Coord     h,
-                      const char * str,
-                      int          style,
+                      const char * str,     /* string to draw */
+                      int          style,   /* font style and size */
                       int          size,
-                      FL_COLOR     c,
-                      FL_COLOR     bc,
-                      FL_COLOR     cc,
-                      int          bk,
-                      int          pos )
+                      FL_COLOR     c,       /* color for text */
+                      FL_COLOR     bc,      /* background color */
+                      FL_COLOR     cc,      /* color for cursor */
+                      int          bk,      /* draws background when set */
+                      int          pos )    /* index of cursor position */
 {
-    fli_draw_string( align, x, y, w, h, 0, FL_WHITE, c, cc,
+    fli_draw_string( align, x, y, w, h, 0, FL_NOCOLOR, c, cc,
                      style, size, pos, 0, -1, str, bk, 0, 0, bc );
 }
 
 
-#define D( x, y, c )                                      \
-    fli_draw_text_cursor( align, x, y, w, h, str,         \
-                          style,size, c, bc, 0, bk, -1 )
+/***************************************
+ * Draws a (multi-line) text with a cursor (no background, no highlighting)
+ ***************************************/
+
+void
+fl_draw_text_cursor( int          align,   /* alignment in box */
+                     FL_Coord     x,       /* box geometry */
+                     FL_Coord     y,
+                     FL_Coord     w,
+                     FL_Coord     h,
+                     FL_COLOR     c,       /* text color */
+                     int          style,   /* font style and size */
+                     int          size,
+                     const char * str,     /* the text to draw */
+                     FL_COLOR     cc,      /* cursor color */
+                     int          pos )    /* cursor position */
+{
+    fli_draw_text_cursor( align, x, y, w, h, str, style, size,
+                          c, FL_NOCOLOR, cc, 0, pos );
+}
 
 
 /***************************************
  ***************************************/
 
+#define D( x, y, c )                                      \
+    fli_draw_text_cursor( align, x, y, w, h, str,         \
+                          style, size, c, bc, 0, bk, -1 )
+
 void
 fli_draw_text_inside( int          align,
                       FL_Coord     x,
@@ -842,7 +878,11 @@ fli_draw_text_inside( int          align,
     /* Take care of special effects stuff  */
 
     if ( special == FL_SHADOW_STYLE )
+#if FL_ENABLE_XFT
+        D( x + 2, y + 2, FL_RIGHT_BCOL );
+#else
         D( x + 2, y + 2, FL_BOTTOM_BCOL );
+#endif
     else if ( special == FL_ENGRAVED_STYLE )
     {
         D( x - 1, y,     FL_RIGHT_BCOL );
@@ -863,7 +903,7 @@ fli_draw_text_inside( int          align,
     }
 
     fli_draw_text_cursor( align, x, y, w, h, str, style, size,
-                          c, bc, 0, special ? 0 : bk, -1 );
+                          c, bc, FL_NOCOLOR, special ? 0 : bk, -1 );
 }
 
 
@@ -886,66 +926,6 @@ fl_draw_text( int          align,
 }
 
 
-/***************************************
- ***************************************/
-
-#if 0
-
-void
-fl_draw_text_beside( int        align,
-                     FL_Coord   x,
-                     FL_Coord   y,
-                     FL_Coord   w,
-                     FL_Coord   h,
-                     char     * str,
-                     int        len,
-                     int        style,
-                     int        size,
-                     FL_COLOR   c,
-                     FL_COLOR   bc,
-                     int        bk )
-{
-    int newa,
-        newx,
-        newy,
-        dx = 0,
-        dy = 0;
-
-    if ( ! str || ! *str )
-        return;
-
-    if ( fl_is_inside_lalign( align ) && ! fl_is_center_lalign( align ) )
-        M_warn( "drw_text_beside", "align request is inside" );
-
-    if ( align & FL_ALIGN_LEFT )
-    {
-        if ( align & FL_ALIGN_BOTTOM || align & FL_ALIGN_TOP )
-            dx = - 4;
-        else
-            dx = 1;
-    }
-    else if ( align & FL_ALIGN_RIGHT )
-    {
-        if ( align & FL_ALIGN_BOTTOM || align & FL_ALIGN_TOP )
-            dx = 4;
-        else
-            dx = - 1;
-    }
-
-    if ( align & FL_ALIGN_BOTTOM )
-        dy = - 2;
-    else if ( align & FL_ALIGN_TOP )
-        dy = 2;
-
-    x += dx;
-    y += dy;
-
-    fli_get_outside_align( align, x, y, w, h, &newa, &newx, &newy );
-    fli_draw_text_inside( align, x, y, w, h, str, style, size, c, bc, bk );
-}
-#endif
-
-
 /***************************************
  ***************************************/
 
@@ -1022,24 +1002,37 @@ fli_set_ul_property( int prop,
  ***************************************/
 
 XRectangle *
-fli_get_underline_rect( XFontStruct * fs,
+fli_get_underline_rect(
+#if FL_ENABLE_XFT
+                        XftFont     * fs,
+#else
+                        XFontStruct * fs,
+#endif
                         FL_Coord      x,
                         FL_Coord      y,
-                        const char  * cstr,
+                        const char  * str,
                         int           n )
 {
     static XRectangle xr;
     int ul_width,
         ul_rwidth,
         xoff;
-    unsigned long ul_pos,
+    unsigned long ul_pos  = 0,
                   ul_thickness = 0;
-    char *str = ( char * ) cstr;
     int ch = *( str + n );
     int pre;                /* stuff in front of the string, such as ^H */
+#if FL_ENABLE_XFT
+    XGlyphInfo extents;
 
+    XftTextExtents8( fl_display, fs, ( const XftChar8 * ) "_", 1, &extents );
+    ul_thickness = extents.height;
+
+    XftTextExtents8( fl_display, fs,
+                     ( const XftChar8 * ) str + n, 1, &extents );
+    ul_pos = extents.height - extents.y + ul_thickness / 4 + 1;
+#else
     if ( UL_thickness < 0 )
-        XGetFontProperty( flx->fs, XA_UNDERLINE_THICKNESS, &ul_thickness );
+        XGetFontProperty( fs, XA_UNDERLINE_THICKNESS, &ul_thickness );
     else
         ul_thickness = UL_thickness;
 
@@ -1047,14 +1040,15 @@ fli_get_underline_rect( XFontStruct * fs,
         ul_thickness = strstr( fli_curfnt, "bold" ) ? 2 : 1;
 
     if ( ! XGetFontProperty( fs, XA_UNDERLINE_POSITION, &ul_pos ) )
-        ul_pos = DESC( ch ) ? ( 1 + flx->fdesc ) : 1;
+        ul_pos = 1 + ( DESC( ch ) ? flx->fdesc : 0 );
+#endif
 
-    /* If the character is narrow, use the width of g otherwise use the width
+    /* If the character is narrow, use the width of h otherwise use the width
        of D. Of course, if UL_width == proportional, this really does not
        matter */
 
-    ul_width = XTextWidth( fs, NARROW( ch ) ? "h" : "D", 1 );
-    ul_rwidth = XTextWidth( fs, str + n, 1 );
+    ul_width  = fli_get_string_width( fs, NARROW( ch ) ? "h" : "D", 1 );
+    ul_rwidth = fli_get_string_width( fs, str + n, 1 );
 
     pre = str[ 0 ] == *fl_ul_magic_char;
 
@@ -1119,17 +1113,36 @@ do_underline_all( FL_Coord        x,
         return;
 
     if ( UL_thickness < 0 )
-        XGetFontProperty( flx->fs, XA_UNDERLINE_THICKNESS, ul_thickness );
+#if FL_ENABLE_XFT
+    {
+        XGlyphInfo extents;
+
+        XftTextExtents8( fl_display, flx->fs, ( const XftChar8 * ) "_", 1,
+                         &extents );
+        *ul_thickness = extents.height;
+    }
+#else
+    	XGetFontProperty( flx->fs, XA_UNDERLINE_THICKNESS, ul_thickness );
+#endif
     else
         *ul_thickness = UL_thickness;
 
     if ( *ul_thickness == 0 || *ul_thickness > 100 )
         *ul_thickness = strstr( fli_curfnt, "bold" ) ? 2 : 1;
 
+#if FL_ENABLE_XFT
+    {
+        XGlyphInfo extents;
+        XftTextExtents8( fl_display, flx->fs,
+                         ( const XftChar8 * ) str, n, &extents );
+        *ul_pos = extents.height - extents.y + *ul_thickness / 4 + 1;
+    }
+#else
     if ( ! XGetFontProperty( flx->fs, XA_UNDERLINE_POSITION, ul_pos ) )
         *ul_pos = has_desc( str ) ? ( 1 + flx->fdesc ) : 1;
+#endif
 
-    ul_width = XTextWidth( flx->fs, str, n );
+    ul_width = fli_get_text_width( str, n );
 
     /* Draw it */
 
@@ -1140,12 +1153,12 @@ do_underline_all( FL_Coord        x,
 
 
 /***************************************
- * Draw a single line string possibly with embedded tabs
+ * Draw a single line string with embedded tabs
  ***************************************/
 
 int
-fli_draw_stringTAB( Window       win,
-                    GC           gc,
+fli_draw_stringTAB( Drawable     win,
+                    FL_COLOR     color,
                     int          x,
                     int          y,
                     int          style,
@@ -1157,30 +1170,95 @@ fli_draw_stringTAB( Window       win,
     int w, tab;
     const char *p,
                *q;
-    XFontStruct *fs = fl_get_font_struct( style, size );
+#if FL_ENABLE_XFT
+    XftFont * fs;
+#else
+    GC gc;
+    XFontStruct * fs;
     DrawString drawIt = img ? XDrawImageString : XDrawString;
+    FL_COORD cx, cy, cw, ch;
+#endif
 
-    if ( win == 0 )
+    if ( win == None )
         return 0;
 
+#if FL_ENABLE_XFT
+    fs = fl_get_font_struct( style, size );
+#else
+    gc = XCreateGC( flx->display, win, 0, NULL );
+    fs = fl_get_font_struct( style, size );
+#endif
+
     tab = fli_get_tabpixels( fs );
 
+#if ! FL_ENABLE_XFT
     XSetFont( flx->display, gc, fs->fid );
+    XSetForeground( flx->display, gc, fl_get_flcolor( color ) );
+
+    fl_get_text_clipping( 1, &cx, &cy, &cw, &ch );
+    if ( cw > 0 || ch > 0 )
+        fl_set_gc_clipping( gc, cx, cy, cw, ch );
+#endif
 
     for ( w = 0, q = s; *q && ( p = strchr( q, '\t' ) ) && p - s < len;
           q = p + 1 )
     {
+#if FL_ENABLE_XFT
+        draw_string( flx->display, fs, color, x + w, y,
+                     ( const XftChar8 * ) q, s - q, img );
+#else
         drawIt( flx->display, win, gc, x + w, y, ( char * ) q, p - q );
-        w += XTextWidth( fs, q, p - q );
-        w = ( w / tab + 1 ) * tab;
+#endif
+        w = ( ( w + fli_get_string_width( fs, q, p - q ) ) / tab + 1 ) * tab;
     }
 
+#if FL_ENABLE_XFT
+    draw_string( flx->display, fs, color, x + w, y,
+                 ( const XftChar8 * ) q, s - q + len, img );
+#else
     drawIt( flx->display, win, gc, x + w, y, ( char * ) q, s - q + len );
 
+    if ( cw > 0 || ch > 0 )
+        fl_unset_gc_clipping( gc );
+    XFreeGC( flx->display, gc );
+#endif
+
     return 0;
 }
 
 
+/***************************************
+ * Draws an Xft string, possibly with background
+ ***************************************/
+
+#if FL_ENABLE_XFT
+static void
+draw_string( Display        * display,
+             XftFont        * font,
+             FL_COLOR         fg_color,
+             int              x,
+             int              y,
+             const XftChar8 * str,
+             size_t           len,
+             int              draw_background )
+{
+    if ( draw_background )
+    {
+        XGlyphInfo extents;
+
+        XftTextExtents8( display, font, str, len, &extents );
+
+        XftDrawRect( flx->bgdraw, &flx->bktextcolor, x, y,
+                     extents.width, font->height );
+    }
+
+    fli_textcolor( fg_color );
+
+    XftDrawString8( flx->textdraw, &flx->textcolor, font, x, y, str, len );
+}
+#endif
+
+
 /*
  * Local variables:
  * tab-width: 4
